	LOGICAL FUNCTION SCRIPT_FILE(FILES)
C
C	This routine is used to read a script file which contains special
C	commands used for dumping a file to the remote.
C
	INCLUDE 'COM.INC/NOLIST'

	LOGICAL GET_VAXFILE
	CHARACTER*(*) FILES
	CHARACTER*80 WAIT_STR
	CHARACTER*32 SCRIPT_TMO, WAIT_TRYS
	INTEGER*4 B, S, E, STMO, WTRYS, WSIZE, RETRYS
	DATA STMO, WTRYS, WSIZE /1,5,0/

	ASSIGN 50 TO NEXT_FILE		! Here to open the next file.
	ASSIGN 100 TO FLOOP		! Loop reading from file.
	ASSIGN 200 TO RLOOP		! Loop reading from remote.
	ASSIGN 9999 TO ABORT		! Go here to abort.
	SCRIPT_FILE = .FALSE.		! Initialize to bad return.
	CALL FLUSH_LOGFILE()		! Empty the logfile buffer.
	CALL REENABLE()			! Enable CTRL/C AST's.
C
C	Parse the file names (if any).
C
	B = 1				! Start at first character.
	CALL PARSE_CMD(FILES,B,S,E)	! Parse the file name (if any).
50	IF (.NOT. GET_VAXFILE(FILES(S:E-1))) RETURN ! Get/open the VAX file.
C
C	Ready to start the script file.
C
	CALL INIT_TIMER()		! Initialize the timer.
C
C	The symbol SCRIPT_TMO can be defined to override the amount of
C	time to wait for the first character from the remote after we send
C	a command line.  Actually, this timeout only has affect if we're
C	going to a system which doesn't echo the command line such as an
C	IBM system.  Otherwise, we timeout after the idle timeout value in
C	the READ_DATA routine.
C
	IF (GET_SYMBOL('SCRIPT_TMO',SCRIPT_TMO,SIZE)) THEN
	    IF (.NOT. CVT_DTB(SCRIPT_TMO(1:SIZE),STMO)) GO TO ABORT
	ENDIF
C
C	The symbol WAIT_TRYS can be defined to specify the number of times
C	to try and find the specified wait string.
C
	IF (GET_SYMBOL('WAIT_TRYS',WAIT_TRYS,SIZE)) THEN
	    IF (.NOT. CVT_DTB(WAIT_TRYS(1:SIZE),WTRYS)) GO TO ABORT
	ENDIF

100	IF (CONTROLC_TYPED) GO TO ABORT	! If aborted, clean up and return.
	READ (FILE_UNIT,110,END=9900) RBYTES,(XBUFFER(I),I=1,RBYTES)
110	FORMAT (Q,<RBYTES+1>A1)
	XDESC(2) = %LOC(XBUFFER)	! Fill descriptor with address.
	XDESC(1) = RBYTES		! 	and the byte count.
C
C	If the first character is a period (.), we presume this is a 
C	script command.  The only script command currently implemented
C	is the .WAIT string.
C
	IF (XBUFFER(1) .EQ. '.') THEN
	    IF (WAIT_CMD(XDESC,WAIT_STR,WSIZE)) GO TO FLOOP
	ENDIF
C
C	If the first character is an up arrow (^) convert character to
C	a control character.  Otherwise, we append a carraige return.
C
	IF (XBUFFER(1) .EQ. '^') THEN
	    CALL CVT_CTRL(XBUFFER(1),RBYTES)
	ELSE
	    RBYTES = RBYTES + 1		! Adjust the byte count.
	    XBUFFER(RBYTES) = CR	! Append a carriage return.
	ENDIF
	XDESC(1) = RBYTES		! Fill descriptor with count.
C
C	Now we write the buffer to the remote without waiting for the
C	write to complete so we can fall though and start our read.
C
	STATUS = SYS$QIO(%VAL(REFN_OUT),%VAL(RCHAN_OUT),
	1		%VAL(IO$_WRITELBLK + IO$M_NOFORMAT),
	1		XIOSB,,,XBUFFER,%VAL(RBYTES),,,,)
	CALL CHECK_STATUS('SCRIPT_WRITE',STATUS)
C
C	Now, we read the output from the remote system.  The READ_DATA
C	routine returns failure if the initial read timed out, if any
C	system service fails, or if the user typed CTRL/C to abort.
C
	RETRYS = 0			! Initialize the retry count.
200	IF (CONTROLC_TYPED) GO TO ABORT	! Stop sending if we were aborted.
	IF (READ_DATA(RBUFFER(1),.TRUE.,RDESC,STMO)) THEN
	    IF (WSIZE .GT. 0) THEN
		IF (FIND_SUBSTRING(WAIT_STR(1:WSIZE),RDESC) .GT. 0) THEN
		    GO TO FLOOP		! Read the next input line.
		ENDIF
	    ENDIF
	    GO TO RLOOP			! Loop until we're idle.
	ELSE
	    IF (WSIZE .GT. 0) THEN	! Retry if wait string specified.
		RETRYS = RETRYS + 1	! Adjust the retry count.
		IF (RETRYS .LE. WTRYS) GO TO RLOOP
	    ENDIF
	ENDIF
	CALL SYS$WAITFR(%VAL(REFN_OUT))	! Wait for write to complete.
	IF (CONTROLC_TYPED) GO TO ABORT	! Stop sending if we were aborted.
	GO TO FLOOP			! Go read the next input record.
C
C	Here for EOF on input.
C
9900	CALL WRITE_USER(SS//'*** End of script file detected - '//
	1		VAX_FILE(1:VSIZE)//' ***'//SS)

9999	CLOSE (UNIT=FILE_UNIT)		! Close the VAX file.
C
C	Let the user know we detected their CTRL/C to abort.
C
	IF (CONTROLC_TYPED) THEN
		CALL WRITE_USER(SS//
	1		'*** Script file has been aborted. ***'//SS)
	ELSE
		CALL PARSE_CMD(FILES,B,S,E) ! Another script file ?
		IF (S .GT. 0) GO TO NEXT_FILE
	ENDIF
	CALL ELAPSED_TIME		! Display the elapsed time.
	SCRIPT_FILE = .TRUE.		! Show success.
	RETURN
	END

	LOGICAL FUNCTION WAIT_CMD(CMD,WBUF,WSIZ)
C
C	This routine is used to setup the wait for string for the .WAIT
C	command.
C
C	Inputs:
C		CMD = The command line buffer.
C		WBUF = Buffer to store the wait string (if any).
C		WSIZ = The size of the wait string.
C
C	Outputs:
C		Returns .FALSE. if no wait string specified.
C
	CHARACTER*(*) CMD, WBUF, SS
	PARAMETER (SS = CHAR(13)//CHAR(10))
	INTEGER*4 WSIZ, B, E, S

	WSIZ = 0			! Presume no wait string.
	WAIT_CMD = .FALSE.		! Initialize the return status.
	IF (CMD(1:2) .NE. '.W') RETURN
	WAIT_CMD = .TRUE.		! Show wait command was found.
C
C	Use the command parsing routine to find the beginning of the
C	wait string.  This routine uses a space as the delimiter.
C
	B = 3				! Starting command position.
	CALL PARSE_CMD(CMD,B,S,E)	! Try to find a delimiter.
	IF (S .EQ. 0) RETURN		! If EQ, delimiter not found.
	WSIZ = (LEN(CMD) - S) + 1	! Calculate the string size.
	WBUF(1:WSIZ) = CMD(S:LEN(CMD))	! Copy the wait for string.
C
C	*** Debug code ***
C
C	CALL WRITE_USER(SS//'Wait string = '//WBUF(1:WSIZ)//SS)
	CALL CVT_CTRL(%REF(WBUF),WSIZ)
	RETURN
	END
