      PROGRAM PRVMAN
*
*   PRVMAN maintains the database of authorization file of users for the SETPRV
*   program and takes care of encrypting the initial authorization code.
*
*
*   Authorization File
*
*       SYS$PROCEDURE:PRVAUTH.DAT
*
*	    The authorization file contains the information needed by the 
*	SETPRV utility to verify and control privilege usage.
*
*
*   Record definition of Authorization file
*
* Field  Name    Type  Length       Description             
* =====  ====    ====  ======       ===========
*   1    CODE     INT     4   key for indexed access
*   2    NAME     CHR    20   last name of the valid user CHARACTER*20
*   3    CLASS    INT     2   classification of PRIVILEGES authorized
*   4    DAYCODE  BIT     2   code for weekday or weekend usage
*   5    BEFORE   INT     2   privilege can't be used before this time
*   6    AFTER    INT     2   time after which the privileges can't be used
*   7    DEVICE   BIT     8   TT ports where the privileges can be used
*   8    OPERACK  BIT     2   Operator approval required flag
*   9    BTCHNET  BIT     2   Batch / Network SETPRV permission flag
*  10    PRVLST   CHR    80   privilege list for classification .eq. 0
*
*
*
*     CODE
*     ====
*
*         Code is the primary key to the authorization file. This value
*     is produced from the user's password by applying the AUTODIN II CRC
*     polynomial.
*
*
*     NAME
*     ====
*
*	  Name is the secondary key to the authorization file and is the last
*     name of the user or some type of identifying label.
*
*
*     CLASS
*     =====
*
*	  Class is the user's classification level. The user can request any
*     privileges from his designated classification level or from any lower
*     level. If the user's class is zero then the user can not request any 
*     specific privileges but is automatically given the preassigned privileges
*     as listed in 'PRVLST' (see below).
*
*
*            NAME      LEVEL         PRIVILEGES
*            ====      =====         ==========
*
*        NORMAL/GROUP    1     Group,Mount,Netmbx,Tmpmbx
*
*           DEVOUR       2     Acnt,Allspool,Bugcheck,Exquota,Grpnam
*                                   Prmceb,Prmgbl,Prmmbx,Shmem
*
*           SYSTEM       3     Altpri,Oper,Pswapm,Syslck,World
*
*            FILES       4     Diagnose,Sysgbl,Volpro
*
*            ALL         5     Bypass,Cmexec,Cmkrnl,Detach,Log_io,Pfnmap
*                                   Phy_io,Sysnam,Sysprv
*
*
*
*     DAYCODE
*     =======
*
*	  Daycode determines on what days of the week this particular user
*     is permitted to use the SETPRV Utility.
*
*              7   6   5   4   3   2   1   0                                  
*             -------------------------------
*            I 0 I x I x I x I x I x I x I x I
*             -------------------------------
*                  I   I   I   I   I   I   I            
*
*                  S   S   F   T   W   T   M         
*                  U   A   R   H   E   U   O         
*                  N   T   I   U   D   E   N         
*
*              where x is 1 then SETPRV can be used
*                      is 0 then SETPRV can not be used
*
*
*     BEFORE
*     ======
*
*         Before is the time measured in minutes before which SETPRV 
*    can NOT be used.
*
*
*     AFTER
*     =====
*
*         After is the time measured in minutes after which SETPRV 
*    can NOT be used.
*
*
*           Examples:    BEFORE = 480   AFTER = 1020
*                     SETPRV can NOT be run before 8:00 or after 17:00 
*                     SETPRV can be run between 8:00 and 17:00 (5:00 pm)
*
*                        BEFORE = 0     AFTER = 1439
*                     SETPRV can be run between 0:00 and 23:59
*
*
*     DEVICE
*     ======
*
*        DEVICE is an array of 8 bytes which are used to limit the terminal
*     ports from which the user can run the SETPRV Utility. Device(1)
*     corresponds to controller 'A', Device(2) corresponds to controller 'B'...
*     Device(8) corresponds to controller 'H'. The bits in each device byte 
*     correspond to the units on the controller with Bit 0 used for unit 0,
*     Bit 1 used for unit 1,... Bit 7 used for unit 7. Therefore a one in
*     a particular bit position means that this unit on the corresponding
*     controller is allowed to be used to run SETPRV.
*
*
*     OPERACK
*     =======
*
*         OPERACK is the flag which controls operator interaction with the use
*     of SETPRV. It has one of the following four values:
*
*      0   NONE   No operator interaction is required for SETPRV to operate.
*      1   OPER   The operator is required to (dis)approve privilege requests.
*      2   OVER   The operator can optionally override privilege request
*                     failures when they fall outside of the user's limits.
*		      Ie. before BEFORE time, invalid DAYCODE, ...
*      3   ALL    Operator approval is required at all times.
*
*
*     BTCHNET
*     =======
*
*         BTCHNET is the flag which controls access to SETPRV from BATCH
*     and NETWORK processes. It has one of the following four values:
*
*      0   NONETBATCH  SETPRV can only be run interactively.
*      1   BATCH       SETPRV can be run in batch or interactive modes.
*      2   NETWORK     SETPRV can be run in network or interactive modes.
*      3   NETBATCH    SETPRV can be run in network,batch or interactive modes.
*
*
*     PRVLST
*     ======
*
*         For class equal zero PRVLST is the privileges which the user
*      is automatically given. The user can not request any privileges which
*      are not on the list.
*
*
*   Open the privilege authorization data file
*
      OPEN(8,FILE='SYS$PROCEDURE:PRVAUTH',ERR=999,STATUS='OLD'
     -    ,KEY=(1:4:INTEGER,5:24:CHARACTER),CARRIAGECONTROL='NONE'
     -   ,SHARED,ACCESS='KEYED',ORGANIZATION='INDEXED',RECL=128)
*
*   Until the user enters 'EXIT' or ^Z ; display the menu of operations
*   permitted, accept the selection number, verify that it is a validate
*   selection, and then process the request
*
      DO WHILE(ICHOICE .LT. 7)
 909       WRITE(6,888)
           READ(5,777,ERR=909,END=9999)ICHOICE
           CALL LIB$ERASE_PAGE(1,1)
           IF((ICHOICE .LT. 7).AND.(ICHOICE.GT.0))CALL COMMAND(ICHOICE)
      ENDDO
 888  FORMAT(///,15X,' 1. Add a record '
     -         /,15X,' 2. Delete a record '
     -         /,15X,' 3. Change a single record '
     -         /,15X,' 4. List a single record '
     -         /,15X,' 5. List the entire file '
     -         /,15X,' 6. Create a new file '
     -         /,15X,' 7. Exit            '
     -        //,15X,' CHOICE >> ',$)
 777  FORMAT(I)
9999  CLOSE(8)
      STOP
 999  STOP 'ERROR on the open'
      END
      SUBROUTINE COMMAND(CHOICE)
*
*     This subroutine is called to process the user's request.
*
*
*     CHOICE   FUNCTION
*     ======   ========
*        1    Add a record
*        2    Delete a record
*        3    Change a record
*        4    List a single record
*        5    List the entire file
*        6    Create a new file
*        7    Exit
*
      IMPLICIT INTEGER(A-Z)
      INTEGER*4 ERROR
      CHARACTER*12 CMD(4)/'(20char max)',' TO DELETE  ',' TO MODIFY  ',
     -                    ' TO LIST    '/
      CHARACTER NAME1*20,ANS,OPRLST*20,BATNETLST*8
      CHARACTER BEFLST*5,AFTLST*5,DAYLST*30,DEVLST*80
*
*  data record definition
*
      BYTE RECORD(128),DEVICE(8)
      INTEGER*4 CODE
      CHARACTER NAME*20,PRVLST*80
      INTEGER*2 CLASS,BEFORE,AFTER,DAYCODE,OPERACK,BTCHNET
      EQUIVALENCE (RECORD(1),CODE),(RECORD(5),NAME)
     -           ,(RECORD(25),CLASS),(RECORD(27),DAYCODE)
     -           ,(RECORD(29),BEFORE),(RECORD(31),AFTER)
     -           ,(RECORD(41),OPERACK),(RECORD(33),DEVICE) 
     -           ,(RECORD(43),BTCHNET),(RECORD(46),PRVLST)
*
*   Determine the users menu choice and process it
* 
      IF(CHOICE.GE.5) THEN
           IF(ICHOICE.EQ.6) THEN
*
*     CREATE A NEW FILE
*
*          Write out the following message and accept the response.
*              Are you sure you want to create a new file (Y/N)?
*          If the response was 'Y' then close the old file and open a new file
*
                WRITE(6,119)
                READ(5,111,ERR=9999,END=9999)ANS
                IF((ANS.EQ.'Y').OR.(ANS.EQ.'y'))THEN
                     CLOSE(8)
                     OPEN(8,ACCESS='KEYED',CARRIAGECONTROL='NONE'
     -                   ,FILE='SYS$PROCEDURE:PRVAUTH',ERR=9999 
     -                   ,KEY=(1:4:INTEGER,5:24:CHARACTER)
     -                   ,ORGANIZATION='INDEXED',RECL=128,STATUS='NEW')
                ENDIF
           ELSE
*
*     LIST THE ENTIRE FILE
*
*          Determine if the listing should go to a file or to the terminal
*          Set the key (NAME) to the smallest possible value
*          Write out the header and read the first record
*
*
                WRITE(6,201)
                READ(5,111,END=999,ERR=999)ANS
                LUN=6
                IF((ANS.EQ.'Y').OR.(ANS.EQ.'y'))THEN
                     LUN=2
                     OPEN(2,FILE='PRVAUTH.LIS',ERR=9999,STATUS='NEW')
                ENDIF
                NAME='0'
                IOS=0
                WRITE(LUN,200)
                READ(8,KEYGT=NAME,KEYID=1,IOSTAT=IOS,ERR=999)RECORD
*
*          While there are still records in the file to be proccessed     
*               convert the fields in the users record to list format
*               pass the list formatted strings to "LIST" for printing
*               Read the next record
*
                DO WHILE (IOS.EQ.0)
                     CALL GETTIME(BEFORE,'BEFORE',1,BEFLST)
                     CALL GETTIME(AFTER,' AFTER',1,AFTLST)
                     CALL GETDAY(DAYCODE,1,DAYLST)
                     CALL GETOPR(OPERACK,1,OPRLST)
                     CALL GETBATNET(BTCHNET,1,BATNETLST)
                     CALL GETDEV(DEVICE,1,DEVLST)
                     CALL LIST(NAME,CLASS,DAYLST,BEFLST,AFTLST,PRVLST
     -                        ,DEVLST,OPRLST,BATNETLST,LUN)
                     READ(8,IOSTAT=IOS,ERR=999)RECORD
                     IF(IOS.LT.0) RETURN
                ENDDO
*
*    If a lising file was selected then close it
*
                IF(LUN.EQ.2) CLOSE(2)
           ENDIF
      ELSE
*
*   For choices 1-4
*       ADD, DELETE, CHANGE or LIST a record
*
*          Solicit the name of the user record to ADD,DELETE,CHANGE OR LIST
*          Convert the name to uppercase
*
           WRITE(6,100)CMD(CHOICE)
           READ(5,111,ERR=999)NAME1
           ISTAT=STR$UPCASE(NAME1,NAME1)
*
*    For choice 4(LIST) write out the header record
*    mark for all choices that this is not adding a new record
*
           IF(CHOICE.EQ.4) WRITE(6,200)
           IADD=0
           IF(CHOICE.EQ.1) THEN
*
*    For choice 1 (ADD) 
*        mark that a new record will be added to the file
*        set all variables to ther default values
*             all terminal ports are allowed; GROUP(1) class is default;
*             utility is usable between 8:00 AM and 5:00 PM Monday thru Sunday;
*             no operator intervention is required;
*             batch and network operation is permitted; no default privileges
*
                IADD=1
                DO I=1,8
                     DEVICE(I)=255
                ENDDO
                CLASS=1
                BEFORE=480
                AFTER=1020
                DAYCODE=31
                OPERACK=0
                BTCHNET=3
                PRVLST=' '
           ENDIF
           IF((CHOICE.GE.2).AND.(CHOICE.LE.4))THEN
*
*    For choices 2,3,4 DELETE, CHANGE, or LIST a record
*         Read the FIRST record with this secondary key
*         and set the answer(ANS) NO to the question "is this the entry?"
*
 1003           READ(8,KEY=NAME1,KEYID=1,IOSTAT=IOS,ERR=999)RECORD
                ANS='N'
*
*     Because there can be multiple NAMES with different passwords (CODES)
*     in the file it is necessary to show the user each record with the 
*     selected name and for the DELETE and CHANGE choices ask the user to
*     respond "YES" when the desired entry is listed
*
*     Convert the variables to print format and list them out
*
                DO WHILE ((NAME.EQ.NAME1).AND.(ANS.NE.'Y'))
                     CALL GETTIME(BEFORE,'BEFORE',1,BEFLST)
                     CALL GETTIME(AFTER,' AFTER',1,AFTLST)
                     CALL GETDAY(DAYCODE,1,DAYLST)
                     CALL GETOPR(OPERACK,1,OPRLST)
                     CALL GETBATNET(BTCHNET,1,BATNETLST)
                     CALL GETDEV(DEVICE,1,DEVLST)
                     CALL LIST(NAME,CLASS,DAYLST,BEFLST,AFTLST,PRVLST
     -                        ,DEVLST,OPRLST,BATNETLST,6)
*
*     For all choices except LIST ask if this is the desired record
*     For LIST we will just go ahead and print all of them
*
                     IF(CHOICE.LT.4)THEN
                          WRITE(6,101)
                          READ(5,111,ERR=1003)ANS
                          ISTAT=STR$UPCASE(ANS,ANS)
                     ENDIF
*
*     if this is not the desired record then read the next one if one exists
*
                     IF(ANS.NE.'Y')READ(8,IOSTAT=IOS,ERR=999)RECORD
                ENDDO
                IF(ANS.NE.'Y')RETURN
*
*     for choice 2 (DELETE) delete the record and return
*
                IF(CHOICE.EQ.2)THEN
                     DELETE(8,ERR=997,IOSTAT=IOS)
                     RETURN
                ELSE
*
*     For choice 3 (CHANGE) ask for the new name or <CR> for no change
*
                     WRITE(6,105)
                     WRITE(6,102)NAME
                     READ(5,109,ERR=1003)NCHR,NAME1
                     IF(NCHR.GT.0)ISTAT=STR$UPCASE(NAME,NAME1)
                ENDIF
           ENDIF
*
*   for choices 1 and 3 (ADD and CHANGE) 
*       the name we read earlier is a good name for adding a new record
*       mark an error and get a new password or keep the old one 
*
           IF(CHOICE.EQ.1)NAME=NAME1
           IERROR=-1
           DO WHILE (IERROR.EQ.-1)
                CALL GETPASSWRD(CODE,IADD,ICHNG,IERROR)
           ENDDO
*
*       allow the user to change the variables (even on adding a new record
*             he can change the default values)
*
           CALL GETCLASS(CLASS,PRVLST)
           CALL GETTIME(BEFORE,'BEFORE',0,BEFLST)
           CALL GETTIME(AFTER,' AFTER',0,AFTLST)
           CALL GETDAY(DAYCODE,0,DAYLST)
           CALL GETOPR(OPERACK,0,OPRLST)
           CALL GETBATNET(BTCHNET,0,BATNETLST)
           CALL GETDEV(DEVICE,0,DEVLST)
*
*    ICHNG was returned from GETPASSWORD and is zero if the old password
*          was not changed. The record can then be rewritten. If ICHNG
*          is not zero then the password was changed and so was the primary
*          key. Therefore the old record must be deleted and the new one
*          written.
*
           IF(ICHNG.EQ.0)THEN
                REWRITE(8,ERR=998,IOSTAT=IOS)RECORD
           ELSE
                IF(CHOICE.EQ.3) DELETE(8)
                WRITE(8,ERR=998,IOSTAT=IOS)RECORD
           ENDIF
      ENDIF
 999  RETURN
*
*      An error occurred while writing the record. If the error code is 50
*          then a duplicate password was entered. Ask the user for a new one.
*          If the error code is other than 50 then tell him he screwed up.
*
 998  IF(IOS .EQ. 50)THEN
           WRITE(6,800)
           CALL GETPASSWRD(CODE,IADD,ICHNG,IERROR)
           WRITE(8,ERR=998,IOSTAT=IOS)RECORD
      ELSE
           WRITE(6,900)IOS
      ENDIF
      RETURN
*
*     error occurred while trying to open prvauth.dat
*    
9999  STOP 'ERROR on the open'
*
*     error occurred while trying to delete a record
*
 997  WRITE(6,225)IOS
      RETURN
 100  FORMAT(2X,'Enter the name of the user ',A,'>> '$)
 102  FORMAT(2X,' Name of USER is ',A14,' >> ',$)
 101  FORMAT('  IS THIS THE ENTRY >> ',$)
 105  FORMAT('  TYPE RETURN TO LEAVE ENTRY UNCHANGED '//)
 109  FORMAT(Q,A)
 111  FORMAT(A)
 119  FORMAT(2X,'Are you sure you want to create a new file (Y/N)?',$)
 200  FORMAT(//2X,'Name         Class    Daycode        Before
     -  After         Flags        Devices'//)
 201  FORMAT(2X,'To listing file PRVAUTH.LIS (Y/N)?',$)
 225  FORMAT(2X,'ERROR record not deleted ',I)
 800  FORMAT(2X,'Duplicate key use another password ')
 900  FORMAT(2X,'Error on writing prvauth ',I6) 
      END
      SUBROUTINE LIST(NAME,CLASS,DAYLST,BEFLST,AFTLST,PRVLST,DEVLST,
     -                OPRLST,BATNETLST,LUN)
*
*     SUBROUTINE LIST
*     ===============
*
*          The purpose of this subroutine is to PRINT the variables
*          of a selected record in the desired format.
*
*
*     INPUT VARIABLES
*     ===== =========
*         NAME   The name of the user for this record.
*         CLASS  The class number that this user falls into.
*         *LST   The variable list items.
*         lun    The logical unit number of the device for printing.
*
*     OUTPUT VARIABLES
*     ====== =========
*           (NONE)
*
*
      INTEGER*2 CLASS,LUN
      CHARACTER NAME*20,PRVLST*80,BEFLST*5,AFTLST*5
      CHARACTER DAYLST*30,DEVLST*80,OPRLST*20,BATNETLST*8
      WRITE(LUN,100)NAME,CLASS,DAYLST,BEFLST,AFTLST,OPRLST
     -,BATNETLST,DEVLST
      IF(CLASS.EQ.0)WRITE(LUN,101)PRVLST
  100 FORMAT(1X,A16,I1,3X,A18,1X,A5,2X,A5,2X,2X,A4,2X,A8,2X,A32)
  101 FORMAT(1X,'Priv list : ',A80)
      RETURN
      END
      SUBROUTINE GETPASSWRD(CODE,IADD,ICHNG,ERROR)
*
*     SUBROUTINE GETPASSWORD
*     ======================
*
*          The purpose of this subroutine is to accept a password from
*          the user and to convert the input string to its crc CODE.
*
*
*     INPUT VARIABLES
*     ===== =========
*         CODE    Current coded password
*         IADD    Logical variable if 0 then not adding else adding new record
*
*     OUTPUT VARIABLES
*     ====== =========
*         CODE    New coded password
*         ICHNG   Logical variable which is 0 if password was not changed
*         ERROR   Logical variable which is 1 if an error is detected
*
      CHARACTER BUFF1*30,BUFF*30,TERM*11 /'SYS$COMMAND'/
      INTEGER*2 IOSB(4),SYS$QIOW,TTCHAN,I_QIOSTAT
      INTEGER*4 CHAN,FUNC,TABLE(16),CODE,ERROR,IADD
      INCLUDE '($IODEF)'
*
*      Initialize the crc table and assign "SYS$COMMAND" to input channel
*      set the qiow function code for read with noecho and uppercase convert
*
      CALL LIB$CRC_TABLE('EDB88320'X,TABLE)
      CALL SYS$ASSIGN(TERM,TTCHAN,,)
      FUNC= IO$_READVBLK .OR. IO$M_NOECHO .OR. IO$M_CVTLOW 
*
*     While an unsatisfactory answer from the user exists
*
      ERROR=1
      DO WHILE (ERROR .EQ.1)
*
*       Mark that the password has not been changed; write out the prompt
*       read the password using a nonechoed QIOW
*       stop if there is a directive error
*
           ICHNG=0
           WRITE(6,201)
           BUFF=' '
           BUFF1=' '
           I_QIOSTAT=SYS$QIOW(,%VAL(TTCHAN),%VAL(FUNC),IOSB,,,
     -                        %REF(BUFF),%VAL(LEN(BUFF)),,,,)
           IF(.NOT. I_QIOSTAT)CALL LIB$STOP(%VAL(I_QIOSTAT))
           IF (IOSB(2).GT.0)THEN
*
*      If a password was entered then prompt for verification
*      Read the password a second time again with a nonechoed QIOW
*      stop if a directive error occurs
*
                WRITE(6,'(23X,A,$)')'Verification->> '
                I_QIOSTAT=SYS$QIOW(,%VAL(TTCHAN),%VAL(FUNC),,,,
     -                            %REF(BUFF1),%VAL(LEN(BUFF1)),,,,)
                IF(.NOT. I_QIOSTAT)CALL LIB$STOP(%VAL(I_QIOSTAT))
                IF(BUFF1 .EQ. BUFF)THEN
*
*       if the first password entered is the same as the verification password
*            then perform AUTODIN II on the buffer and mark that CODE has
*            changed and no error condition exists
*
                     CODE=LIB$CRC(TABLE,-1,%DESCR(BUFF))
                     CODE=.NOT. CODE
                     ICHNG=1
                     ERROR=0
                ENDIF
           ELSE
*
*      if a <CR> was entered and if this is a CHANGE selection then 
*        the password stays the same and we can return with no error
*      If a <CR> was entered and this is a ADD selection then repeat DO
*
                IF (IADD.EQ.0)ERROR=0
           ENDIF
      ENDDO
      RETURN
 201  FORMAT(2x,'Enter the user 1-30 char password >> '$)
      END
      SUBROUTINE GETOPR(OPRACK,ITYPE,OPRLST)
*
*     SUBROUTINE GETOPR
*     =================
*
*          The purpose of this subroutine is to convert the OPRACK flag to its
*          corresponding text string(OPRLST) and optionally accept a new value.
*
*
*     INPUT VARIABLES
*     ===== =========
*         OPRACK  Current operator acknowledgement flag
*         ITYPE   =0 then accept new value else just return character string
*
*     OUTPUT VARIABLES
*     ====== =========
*         OPRACK  New operator acknowledgement flag
*         OPRLST  Operator acknowledgement character text string equivalence
*
*
      CHARACTER*4 OPRLST,LIST(4),INPUT
      INTEGER*2 OPRACK
      INTEGER*4 ITYPE,N
      BYTE B(4)
      EQUIVALENCE (B,INPUT)
*
*      Possible values of OPRLST which correspond to OPRACK=0,1,2,3
*
      DATA LIST/'NONE','OPER','OVER','ALL '/
*
*       If OPRACK is not valid then set it to 0("NONE")
*       OPRLST gets the text string which corresponds to OPRACK
*
      IF((OPRACK.LT.0).OR.(OPRACK.GT.3))OPRACK=0
      OPRLST=LIST(OPRACK+1)
*
*  For cases where the value of OPRACK can be changed (ITYPE=0) 
*     While an unsatisfactory answer from the user exists
*            Show the user the current value and accept a new value 
*            If a new value is entered convert it to uppercase else return old
*
      IF(ITYPE.EQ.0)THEN
           ERROR=1
           DO WHILE (ERROR .EQ. 1)
                WRITE(6,19)OPRLST
                READ(5,29,ERR=99)N,B
                IF(N.GT.0)ISTAT=STR$UPCASE(INPUT,INPUT)
                IF(N.LE.0) RETURN
*
*         Compare the new value to the list of legal values
*           If a match is found then update OPRACK and OPRLST
*           set no error detected and return
*
                DO I=1,4
                     IF(INPUT(1:N).EQ.LIST(I)(1:N))THEN
                          OPRACK=I-1
                          OPRLST=LIST(I)
                          ERROR=0
                     ENDIF
                ENDDO
           ENDDO
      ENDIF
 99   RETURN
  19  FORMAT(2x,' Operator reply flag (NONE,OPER,OVER,ALL) is [',A4,
     -       '] >>',$)
  29  FORMAT(Q,4A1)
      END
      SUBROUTINE GETTIME(BEFAFT,TITLE,ITYPE,BALST)
*
*     SUBROUTINE GETTIME
*     ==================
*
*          The purpose of this subroutine is to convert BEFAFT time to its
*          corresponding text string(BALST) and optionally accept a new value.
*
*
*     INPUT VARIABLES
*     ===== =========
*         BEFAFT  Current Before/After time value
*         TITLE   Name of this variable "BEGIN" or "END"
*         ITYPE   =0 then accept new value else just return character string
*
*     OUTPUT VARIABLES
*     ====== =========
*         BEFAFT  Current Before/After time value
*         BALST   Text string corresponding to BEFAFT
*
*
      CHARACTER TITLE*6,BALST*5,BAL*5
      BYTE B(5)
      INTEGER*2 BEFAFT,I,J,J1,J2,N,ERROR
      EQUIVALENCE (BAL,B)
*
*     Convert BEFAFT time to hours(I) and minutes(J)
*     Convert I,J to an ascii text string with a ":" in the middle
*     The array B and BAL are equivalenced BALST recieves the value in BAL.
*
      I=BEFAFT/60
      J=BEFAFT-I*60
      B(1)=I/10+48
      B(2)=I-I/10*10+48
      B(3)=':'
      B(4)=J/10+48
      B(5)=J-J/10*10+48
      BALST=BAL
*
*  For cases where the value of BEFAFT can be changed (ITYPE=0) 
*     While an unsatisfactory answer from the user exists
*        Prompt the user for a before/after time and accept the response
*        if the response was just a <CR> then return with the current value
*
      IF(ITYPE.EQ.0)THEN
           ERROR=1
           DO WHILE (ERROR .EQ. 1)
                WRITE(6,19)TITLE,BAL
                READ(5,29,ERR=99)N,B
                IF(N.LE.0) RETURN
*
*          Set hours and minutes to zero along with the error and colon flags
*
                MIN=0
                IHR=0
                IERR=0
                ICOLON=0
*
*         For the "N" characters that where read in from the terminal
*             after 2 characters the colon flag is set to select minutes
*             each character read must be between 48 ("0") and 57 ("9") or
*                a colon (":") else it is an ERROR
*
                DO I=1,N
                     IF(I.EQ.3)ICOLON=1
                     IF((B(I).LT.48).OR.(B(I).GT.57))THEN
                          IF(B(I).EQ.':')THEN
                               ICOLON=1
                          ELSE
                               IERR=1
                          ENDIF
                     ELSE
*
*        legal numeric character
*             if colon flag is not set then this is HOURS else its MINUTES
*
                          IF(ICOLON.EQ.0)THEN
                               IHR=IHR*10+B(I)-48
                          ELSE
                               MIN=MIN*10+B(I)-48
                          ENDIF
                     ENDIF
                ENDDO
*
*       if not legal value for hours or minutes then set IERR flag
*
                IF((IHR.LT.0).OR.(IHR.GT.23))IERR=1
                IF((MIN.LT.0).OR.(MIN.GT.59))IERR=1
*
*     if the IERR flag is not set then compute BEFAFT time and clear ERROR
*         else write an error message and reask user for new time
*
                IF(IERR.EQ.0)THEN
                     BEFAFT=IHR*60+MIN
                     ERROR=0
                ELSE
                     WRITE(6,39)B
                ENDIF
           ENDDO
      ENDIF
 99   RETURN
  19  FORMAT(2x,'NOT ',A6,' time is [',A,'] >> '$)
  29  FORMAT(Q,5A1)
  39  FORMAT('  Time Specification Error [',6A1,']')
      END
      SUBROUTINE GETCLASS(CLASS,PRVLST)
*
*     SUBROUTINE GETCLASS
*     ===================
*
*          The purpose of this subroutine is to accept a new classification
*          level and if that level is zero then to accept the specified
*          privileges.
*
*
*     INPUT VARIABLES
*     ===== =========
*         CLASS   This is the current privilege classification level.
*
*     OUTPUT VARIABLES
*     ====== =========
*         CLASS   This is the new privilege classification level.
*         PRVLST  For CLASS equal 0 this is the list of privileges.
*
*
      CHARACTER*80 PRVLST,PRVL,PRV*8,C*1
      INTEGER*2 CLASS,ERROR,N,CLASS1
      BYTE B(8),A
      EQUIVALENCE (B,PRV),(A,C)
*
*     While an unsatisfactory answer from the user exists
*         Prompt for and accept a new classification level
*         if the user didn't enter a <CR> only then set CLASS = new response
*         if the CLASS is within limits then clear ERROR and exit this loop
*
      ERROR=1
      DO WHILE (ERROR.EQ.1)
           WRITE(6,301)CLASS
           READ(5,229,ERR=99)N,CLASS1
           IF(N.GT.0)CLASS=CLASS1
           IF((CLASS .GE. 0).AND. (CLASS .LE. 5))ERROR=0
      ENDDO
*
*      For classification level 0 no privileges can be requested from SETPRV
*          only the predetermined privileges are given -- prompt and accept
*          the privileges from the user. Convert them to uppercase
*
      IF(CLASS .EQ. 0)THEN
           WRITE(6,117)PRVLST
           READ(5,109,ERR=99)NCHR,PRVL
           IF(NCHR.GT.0)ISTAT=STR$UPCASE(PRVL,PRVL)
*
*     Mark that no ERROR has been found and that we are not DONE
*     set the buffer pointer(L) for PRVLST to 0 and (J) for input buffer to 1
*
           ERROR=0
           IDONE=0
           L=0
           J=1
*
*       Until we are done check for a blank in the input buffer(PRVL)
*          a blank means end of buffer so mark it and clear the rest
*
           DO WHILE (IDONE.EQ.0)
                IF(PRVL(J:J).EQ.' ')THEN
                     IDONE=1
                     IF(L.GT.0)PRVLST(L:80)=' '
                ELSE
*
*        extract one privilege at a time from the input buffer and store
*           it in PRV
*
                     PRV=' '
                     I=0
                     DO WHILE((PRVL(J:J).NE.',').AND.(PRVL(J:J).NE.' '))
                          I=I+1
                          C=PRVL(J:J)
                          B(I)=A
                          J=J+1
                     ENDDO
*
*    bump the input pointer past the comma or blank and call GETPRV to
*      return the full privilege name, the number of chars, and if no
*      error condition is returned then append the returned privilege(PRV)
*      to the privilege list(PRVLST) and tack on a "," to the end
*
                     J=J+1
                     CALL GETPRV(PRV,NCHR,ERROR)
                     IF(ERROR.EQ.0)THEN
                          DO N=1,NCHR
                               L=L+1
                               PRVLST(L:L)=PRV(N:N)
                          ENDDO
                          L=L+1
                          PRVLST(L:L)=','
                     ENDIF
                ENDIF
           ENDDO
      ELSE
*
*    For non class 0 levels blank PRVLST because it does not apply
*
           PRVLST=' '
      ENDIF     
   99 RETURN
 109  FORMAT(Q,A)
 117  FORMAT(2x,'Privileges are: '/,4X,A75,/' >> ',$)
 229  FORMAT(Q,I6)
 301  FORMAT(2X,'Class is ',I3,' >> ',$)
      END
      SUBROUTINE GETDAY(DAYCODE,ITYPE,DAYLST)
*
*     SUBROUTINE GETDAY
*     =================
*
*          The purpose of this subroutine is to convert DAYCODE into its
*          corresponding text string and return it in DAYLST and to optionally
*          accept a new value for DAYCODE.
*
*
*     INPUT VARIABLES
*     ===== =========
*         DAYCODE  This variable holds the current allowed days of the week
*         ITYPE   =0 then accept new value else just return character string
*
*     OUTPUT VARIABLES
*     ====== =========
*         DAYCODE  This variable holds the new allowed days of the week
*         DAYLST   Text string corresponding to the value of DAYCODE
*
*
      INTEGER*2 DAYCODE,BIT(9),DAYCD
      INTEGER*4 ITYPE
      CHARACTER*3 DAYS(9),TMP,DAYLST*30
      BYTE B(30),DAYB(27),TMPB(3)
      EQUIVALENCE (DAYB,DAYS),(TMPB,TMP)
      DATA DAYS/'MON','TUE','WED','THU','FRI','SAT','SUN','M-F','ALL'/
      DATA BIT/1,2,4,8,16,32,64,31,127/
*
*     set the number of characters in DAYLST to 0 (JJ) and blank DAYLST
*     mark no error and save current value of DAYCODE in DAYCD
*
      JJ=0
      DAYLST=' '
      IERROR=0
      DAYCD=DAYCODE
      IF(DAYCODE.EQ.127)THEN
*
*     for DAYCODE = 127 then DAYLST is "ALL" and 3 characters long
*
           DAYLST(1:3)=DAYS(9)
           JJ=3
      ELSE
           IF((DAYCD.AND.31).EQ.31)THEN
*
*     for DAYCODE with a 31 then DAYLST is "M-F" and 3 characters long
*     remove the monday-friday bits
*
                DAYCD=DAYCD-31
                DAYLST(1:3)=DAYS(8)
                JJ=3
           ENDIF
*
*    for all days of the week
*        get corresponding bit position and check to see if that bit is
*        on in DAYCD; if it is then append the day text to text string DAYLST
*
          DO I=1,7
                K=2**(I-1)
                IF((DAYCD.AND.K).NE.0)THEN
                     IF(JJ.EQ.0)THEN
                          DAYLST(1:3)=DAYS(I)
                          JJ=3
                     ELSE
                          DAYLST=DAYLST(1:JJ)//','//DAYS(I)
                          JJ=JJ+4
                     ENDIF
                ENDIF
           ENDDO
      ENDIF
*
*  For cases where the value of DAYCODE can be changed (ITYPE=0) 
*       prompt for and accept the new value ; if no value is entered 
*       then the current value is unchanged
*
      IF(ITYPE.EQ.0)THEN
           WRITE(6,19)DAYLST
           READ(5,29,ERR=99)N,B
           IF(N.GT.0)THEN
*
*      since the user has entered a new value; set not end and no days
*         allowed and set buffer pointer(J) to zero
*
                IEND=0
                DAYCD=0
                J=0
*
*       While there are still characters to process
*             get the next three characters and convert them to uppercase
*
                DO WHILE (IEND.EQ.0)
                     DO I=1,3
                          J=J+1
                          TMPB(I)=B(J)
                     ENDDO
                     ISTAT=STR$UPCASE(TMP,TMP)
                     K=0
*
*       find the day code that matches and set the appropriate bits
*           if there is no match then write out an eror message and set error
*                if there is a match then reflect it in DAYCD
*
                     DO I=1,9
                          IF(TMP.EQ.DAYS(I))K=K.OR.BIT(I)
                     ENDDO
                     IF(K.EQ.0)THEN
                          WRITE(6,119)TMP
                          IERROR=1
                     ELSE
                          DAYCD=DAYCD.OR.K
                     ENDIF
*
*      if the buffer is not empty then skip the terminator else mark end
*      if no errors then set DAYCODE equal to DAYCD
*
                     DO WHILE ((J.LT.N).AND.(B(J).NE.','))
                          J=J+1
                     ENDDO
                     IF(J.GE.N)IEND=1
                ENDDO
                IF(IERROR.EQ.0)DAYCODE=DAYCD
           ENDIF
      ENDIF
   99 RETURN
   19 FORMAT(2X,'Days allowed are ',A,/,2X,'  >> ',$)
   29 FORMAT(Q,30A1)
  119 FORMAT(2X,'Unrecognizable day -- [',A,']')
      END
      SUBROUTINE GETPRV(PRIV,NCHR,IERROR)
*
*     SUBROUTINE GETPRV
*     =================
*
*          The purpose of this subroutine is to check that PRIV is a valid
*          privilege name and to return PRIV containing the full spelling
*          and the number of characters(NCHR) contained in priv.
*
*
*     INPUT VARIABLES
*     ===== =========
*         PRIV    Text string containing the privilege to be validated
*
*     OUTPUT VARIABLES
*     ====== =========
*         PRIV    Text string containing the full spelling of the privilege
*         NCHR    Number of characters in PRIV
*         IERROR  Error flag
*
*
      INTEGER*4 IERROR
      INTEGER*2 FOUND,DONE
      INTEGER*2 MIN(30),MAX(30),BIT(30)
      CHARACTER*8 PRV(30),PRIV
      DATA PRV/'ACNT    ','ALLSPOOL','ALTPRI  ','BUGCHK  '
     -        ,'BYPASS  ','CMEXEC  ','CMKRNL  ','DETACH  '
     -        ,'DIAGNOSE','EXQUOTA ','GROUP   ','GRPNAM  '
     -        ,'LOG_IO  ','MOUNT   ','NETMBX  ','OPER    '
     -        ,'PFNMAP  ','PHY_IO  ','PRMCEB  ','PRMGBL  '
     -        ,'PRMMBX  ','PSWAPM  ','SHMEM   ','SYSGBL  '
     -        ,'SYSLCK  ','SYSNAM  ','SYSPRV  ','TMPMBX  '
     -        ,'VOLPRO  ','WORLD   '/
      DATA MIN/2,3,3,2,2,3,3,3*2,3,3,6*2,3*4,2,2,4*4,3*2/
      DATA MAX/4,8,6*6,8,7,5,6,6,5,6,4,6*6,5,6*6,5/
*
*      set that the privilege has not yet been found and that no error exists
*
      FOUND=0
      IERROR=0
      J=0
*
*     Until the privilege is found in the list or we run out of list
*         set pointer to next privilege; set character pointer to beginning
*
      DO WHILE ((J.LE.30) .AND. (FOUND.EQ.0))
           J=J+1
           I=0
           DONE=0
*
*      Until we are done
*          point at next character; compare characters
*
           DO WHILE (DONE.EQ.0)
                I=I+1
                IF(PRV(J)(I:I).NE.PRIV(I:I))THEN
*
*      the characters are not the same
*         set done; if the input character is a blank then if the minimum
*         number of characters have been detected then abbreviated privilege
*               so mark it found and return the full priv and number of chrs
*               else mark it as an error
*
                     DONE=1
                     IF(PRIV(I:I).EQ.' ')THEN
                          IF(MIN(J).LT.I) THEN
                               FOUND=1
                               PRIV=PRV(J)
                               NCHR=MAX(J)
                          ELSE
                               IERROR=2
                          ENDIF
                     ELSE
                          IF(MAX(J).LT.I) FOUND=1
                     ENDIF
                ENDIF
           ENDDO
      ENDDO
*
*      if no privilege was found then mark an error
*      if in error then write an appropriate error message
*
      IF((FOUND.EQ.0).AND.(IERROR.EQ.0))IERROR=1
      IF(IERROR.EQ.2)WRITE(6,19)PRIV
      IF(IERROR.EQ.1)WRITE(6,29)PRIV
   19 FORMAT(' Ambiguous Reference ',A)
   29 FORMAT(' Illegal Privilege ',A)
      RETURN
      END
      SUBROUTINE GETDEV(DEVICE,ITYPE,DEVLST)
*
*     SUBROUTINE GETDEV
*     ====================
*
*          The purpose of this subroutine is to interprete the meaning of
*     the DEVICE flag and put the corresponding text string into DEVLST.
*     Optionally depending on ITYPE a new value for DEVICE is accepted from
*     the user.
*
*     INPUT VARIABLES
*     ===== =========
*
*         DEVICE     flag showing the permitted terminal ports
*         ITYPE      flag to determine if new value(DEVICE) should be accepted
*
*     OUTPUT VARIABLES
*     ====== =========
*
*        DEVICE     flag showing new permitted terminal status(ITYPE=0)
*        DEVLST     text string corresponding to DEVICE
*
      INTEGER*4 ITYPE
      CHARACTER*80 DEVLST,TMP*2
      BYTE DEVICE(8),BIT(8),TMPB(2),B(75),D(3,25)
      EQUIVALENCE (TMP,TMPB),(B,D)
      DATA BIT/1,2,4,8,16,32,64,128/
*
*         To determine the text string(DEVLST) which corresponds to the bits
*     set in DEVICE, the DEVICE array must be scanned for consequetive bit
*     strings of 1's or 0's.
*
*     Initially, set bit string type to 0's
*                set last found terminal port to 0
*                set DEVLST character pointer to 0
*
      ION=0
      LAST=0
      K=0
*
*     Do for controller 1(TTAn) to controller 8(TTHn)
*
      DO I=1,8
*
*     Do for unit 0(TTx0) to unit 7(TTx7)
*
           DO J=1,8
                IF((DEVICE(I).AND.BIT(J)).EQ.0) THEN
*
*     If the bit in DEVICE corresponding to this controller and unit is a 0
*         and if we were in a sequence of 1's then the previous port was the
*         end of a sequence
*
                     IF(ION.EQ.1)THEN
*
*     set L to the previous port number
*     set TMPB to ascii controller letter and unit number
*     mark that we are now looking for a sequence of 0's
*
                          L=I*8+J-1
                          TMPB(1)=I+64
                          TMPB(2)=J+47
                          ION=0
*
*     Since we have already listed the port at the beginning of the sequence
*         then if the end of the sequence is the same then only insert a ','
*              else insert a '-' and then the ending port number
                          IF(L.LE.LAST)THEN
                               DEVLST=DEVLST(1:K)//','
                               K=K+1
                          ELSE
                               DEVLST=DEVLST(1:K)//'-'//TMP//','
                               K=K+4
                          ENDIF
                     ENDIF
                ELSE
*
*     If the bit in DEVICE corresponding to this controller and unit is a 1
*
                     IF(ION.EQ.0)THEN
                          LAST=I*8+J
                          ION=1
                          TMPB(1)=I+64
                          TMPB(2)=J+47
                          IF(K.NE.0)THEN
                               DEVLST=DEVLST(1:K)//TMP
                          ELSE
                               DEVLST=TMP
                          ENDIF
                          K=K+2
                     ENDIF
                ENDIF
*
*     End unit loop
*
           ENDDO
*
*     End controller loop
*
      ENDDO
*
*     If sequence is on then the last port(H7) should terminate sequence
*      therefore stick "H7" in the DEVLST
*
      IF(ION.EQ.1)THEN
           L=71
           TMPB(1)=72
           TMPB(2)=55
           IF(L.GT.LAST)THEN
                DEVLST=DEVLST(1:K)//'-'//TMP
                K=K+3
           ENDIF
      ELSE
           DEVLST(K:K)=' '
      ENDIF
      IERROR=0
*
*  For cases where the value of DEVICE can be changed (ITYPE=0) 
*     Prompt for and accept the new device list
*
      IF(ITYPE.EQ.0)THEN
           WRITE(6,19)DEVLST
           READ(5,29,ERR=99)N,B
*
*     if the user entered more than just a <CR> then he must have entered
*        something in the form:  cu,cu-cu,cu 
*            controller unit ,    controller unit -      controller unit " "
*        each specification consists of a controller, a unit , and a terminator
*        so the total number of charactera entered divided by 3 is the number
*          of specifications with the "-" terminator meaning all ports between
*          the current spec and the next spec and the blank meaning last spec
*
           IF(N.GT.0) THEN
                NN=(N+1)/3
*
*       for all entered specs 
*          is the controller between "A" and "H" if not mark and write error
*          is the unit between "0" and "7" if not mark and write error
*          is the terminator a dash, comma or blank if not mark and write error
*
                DO I=1,NN
                     IF((D(1,I).GT.96).AND.(D(1,I).LT.123))
     -                  D(1,I)=D(1,I)-32
                     IF((D(1,I).LT.65).OR.(D(1,I).GT.73))THEN
                          IERROR=1
                          WRITE(6,119)(D(J,I),J=1,3)
                     ENDIF
                     IF((D(2,I).LT.48).OR.(D(2,I).GT.56))THEN
                          IERROR=1
                          WRITE(6,119)(D(J,I),J=1,3)
                     ENDIF
                     IF((D(3,I).NE.' ').AND.(D(3,I).NE.',').AND.
     -                    (D(3,I).NE.'-'))THEN
                          IERROR=1
                          WRITE(6,119)(D(J,I),J=1,3)
                     ENDIF
                ENDDO
*
*       if no error condition exists then reset all allowed devices
*       select the zeroth spec
*
                IF(IERROR.EQ.0)THEN
                     DO I=1,8
                          DEVICE(I)=0
                     ENDDO
                     I=0
*
*       For all entered specs
*            get the next spec(I) get the controller(J) and unit(K) for the
*            Ith spec and set the corresponding unit bit in the controller byte
*            mark one less spec to process
*
                     DO WHILE (NN.NE.0)
                          I=I+1
                          J=D(1,I)-64
                          K=D(2,I)-47
                          DEVICE(J)=DEVICE(J).OR.BIT(K)
                          NN=NN-1
*
*     If the terminator on this spec is a "-" then it is the start of a
*      sequence for which the end of sequence is specified by spec I+1
*            get the next spec(I) get the controller(J1) and unit(K1) for the
*            mark one less spec to process; compute the number of devices in
*             the sequence
*
                          IF((NN.NE.0).AND.(D(3,I).EQ.'-'))THEN
                               I=I+1
                               NN=NN-1
                               J1=D(1,I)-64
                               K1=D(2,I)-47
                               NUMB=(J1-J)*8+K1-K
*
*     For all the devices in this sequence 
*         set the corresponding unit bit in the controller byte
*         when at the end of a controller byte then start with unit 0 on
*         the next controller byte
*
                               DO WHILE (NUMB.GT.0)
                                    DEVICE(J)=DEVICE(J).OR.BIT(K)
                                    K=K+1
                                    IF(K.GT.8)THEN
                                         K=1
                                         J=J+1
                                    ENDIF
                                    NUMB=NUMB-1
                               ENDDO
                          ENDIF
                     ENDDO
                ENDIF
           ENDIF
      ENDIF
   19 FORMAT(2X,' Allowed device ports are:',/,2x,A,/,' >> ',$)
   29 FORMAT(Q,75A1)
  119 FORMAT(2X,'Error in device name [',3A1,']')
   99 RETURN
      END
      SUBROUTINE GETBATNET(BTCHNET,ITYPE,BATNETLST)
*
*     SUBROUTINE GETBATNET
*     ====================
*
*          The purpose of this subroutine is to interprete the meaning of
*     the BTCHNET flag and put the corresponding text string into BATNETLST.
*     Optionally depending on ITYPE a new value for BTCHNET is accepted from
*     the user.
*
*     INPUT VARIABLES
*     ===== =========
*
*        BTCHNET     flag showing batch/network permission
*         ITYPE      flag to determine if new value(BTCHNET) should be accepted
*
*     OUTPUT VARIABLES
*     ====== =========
*
*        BTCHNET     flag showing new batch/network permission status(ITYPE=0)
*       BATNETLST    text string corresponding to BTCHNET
*
*
      CHARACTER*8 BATNETLST,LIST(4),INPUT
      INTEGER*2 BTCHNET
      INTEGER*4 ITYPE,N
      BYTE B(8)
      EQUIVALENCE (B,INPUT)
      DATA LIST/'NONETBAT','BATCH   ','NETWORK ','NETBATCH'/
*
*         BTCHNET must be between 0 and 3 inclusive; if it isn't then make it 0
*     Set the text string variable(BATNETLST) to correspond to BTCHNET
*
      IF((BTCHNET.LT.0).OR.(BTCHNET.GT.3))BTCHNET=0
      BATNETLST=LIST(BTCHNET+1)
*
*  For cases where the value of BTCHNET can be changed (ITYPE=0) 
*     While an unsatisfactory answer from the user exists
*         Output prompt containing possible values and current value
*         Input desired value ; if null then current value is desired
*                               else convert to upper case and validate
*         If an error is detected then print error message
*         If no error then return with new value for BTCHNET
*
      IF(ITYPE.EQ.0)THEN
           ERROR=1
           DO WHILE (ERROR .EQ. 1)
                WRITE(6,19)LIST,BATNETLST
                READ(5,29,ERR=99)N,B
                IF(N.LE.0) RETURN
                IF(N.GT.0)ISTAT=STR$UPCASE(INPUT,INPUT)
                DO I=1,4
                     IF(INPUT(1:N).EQ.LIST(I)(1:N))THEN
                          BTCHNET=I-1
                          BATNETLST=LIST(I)
                          ERROR=ERROR-1
                     ENDIF
                ENDDO
                IF(ERROR.LT.0)THEN
                     ERROR=1
                     WRITE(6,39)
                ENDIF
           ENDDO
      ENDIF
  99  RETURN
  19  FORMAT(2x,' Batch / Network flag (',3(A8,','),A8,') is [',A8,
     -       '] >>',$)
  29  FORMAT(Q,8A1)
  39  FORMAT(' Ambiguous reference')
      END
