       PROGRAM SETPRV
*
*   Program Name SETPRIV
*
*   Author : SSgt. Michael D. Weaver 
*
*            I would like to acknowlege the very significant
*            contributions of Mr. Dave Lennig (DEC software resident)
*            in the conceptual and initial development of this
*            system.
*
*   Date : 15 Feb 1983
*
*   Instructions for the use and compilation can be found in a separate
*   document.
*
*
*    Subroutines called :
*       
*      SYS$CMEXEC  This calls SYS$SETPRV indirectly. CMEXEC is a
*                  system service to elevate the mode to executive
*                  to allow the SETPRV image to be executed by the 
*                  user mode image.
*      INDEX       Locates a substring within a source string
*      LIB$STOP    Routine to print the error return from one of the
*                  system service routines and do a normal image
*                  rundown.
*      OPCOM       Routine to notify any operator ENABLED as CENTRAL
*                  of an invalid useage attempt. Also notifies the
*                  central operator console and places an entry in 
*                  the "OPERATOR.LOG" file.
*      PARSE       Routine to parse the command line.
*      VALIDATE    Routine to check the validity of the request for
*                  privileges. Can return a status of DONE or ERROR.
*                  Will also return a list of authorized privileges
*                  to allow the appropriate mask of privileges to be 
*                  created and placed in MSK.
*      COMPUTE_MASK Routine to convert the alphanumeric representation
*                  of privileges to be turned on or off to the 64 bit
*                  value required by $SETPRV.
*                  
*      
*   Description :
*    
*     This program sets privilege bits in the process header
*   block and uses CMEXEC to accomplish this. For accountability
*   this program also uses OPCOM to send messages to the central console
*   and also any operator enabled as CENTRAL.
*	The technique used to set privileges is based on the characteristic
*   that any image running in EXECUTIVE or KERNEL mode is implicitly granted
*   the privilege SETPRV. SETPRV is also a system service used by VMS DCL (i.e.
*   SET PROCESS/PRIVILEGE=(priv,...)). Normally SETPRV (the privilege) is
*   only used as a flag set in the UAF entry indicating to the system
*   that the process may set any privilege bit. 
* 
*
      EXTERNAL SYS$CMEXEC,SYS$SETPRV
      INTEGER*4 MSK,MSKON,MSKOFF,MSKAUTH,ARGLST(7)
      INTEGER*4 ERROR,ADD,RESET,LEVEL,DAY_NUMB,TIME
      INTEGER*2 SYS$CMEXEC,ISTAT,SYS$GETJPI,ITMLST(14),DAYS(7)
      INTEGER*2 TODAY,NOW,TTPORT,TTCNTRL,TTDEV,OPRPLY,ICLASS
      CHARACTER*132 PRIV,REASON,NAME*20
      BYTE TTNAM(7)
      INCLUDE '($JPIDEF)'
      EQUIVALENCE (ITMLST,ARGLST)
      DATA DAYS/4,8,16,32,64,1,2/
*
*     Set up the argument list to pass to $GETJPI
*
*     Use the function codes for obtaining Authorized privilege mask and
*    the terminal name.
*
      ITMLST(1)=4
      ITMLST(2)=JPI$_AUTHPRIV
      ARGLST(2)=%LOC(MSKAUTH)
      ARGLST(3)=0
      ITMLST(7)=7
      ITMLST(8)=JPI$_TERMINAL
      ARGLST(5)=%LOC(TTNAM)
      ARGLST(6)=0
      ARGLST(7)=0
      ISTAT=SYS$GETJPI(,,,%VAL(%LOC(ITMLST)),,,)
      IF(.NOT. ISTAT)CALL LIB$STOP(%VAL(ISTAT))
C
C	TTDEV     TYPE
C	-----     ----
C	  0      BATCH
C	  1      TT PORT
C	  2      OPER CONSOLE
C	  3	 REMOTE TERMINAL
C
      TTDEV=0
      IF(TTNAM(1).EQ.'T')TTDEV=1
      IF(TTNAM(1).EQ.'O')TTDEV=2
      IF(TTNAM(1).EQ.'R')TTDEV=3
      TTCNTRL=TTNAM(3)-64
*
*     Don't forget we may need the channel also
*
      IPORT=TTNAM(4)-48
      TTPORT=2**IPORT
*
*	BASE DAY WAS WEDNESDAY NOVEMBER 17, 1858
*
      CALL LIB$DAY(DAY_NUMB,,TIME)
      I=DAY_NUMB-DAY_NUMB/7*7+1
      TODAY=DAYS(I)
      NOW=((TIME+50)/100+30)/60
*
*    Call the parse routine to check the command line and return
*    the list of privileges that 
*
      CALL PARSE(REASON,PRIV,MSKON,MSKOFF,LEVEL,ADD,RESET)
      IF(RESET .EQ. 0)THEN
*
*    Based on the settings of command line switch (/RESET) call
*    validate to determine if there is an entry
*    in "SYS$PROCEDURE:PRVAUTH.DAT" which will allow the user
*    to use the program.
*
            CALL VALIDATE(PRIV,NAME,MSKON,LEVEL,TTDEV,TTCNTRL,
     -TTPORT,TODAY,NOW,OPRPLY,ERROR,ICLASS)
            IF(ERROR.NE.0)THEN
*
*  If any error is passed back from validate assume that the error
*  is invalid authorization and stop
*
               CALL OPCOM(13,' Setprv_E_Invauth  ',)
               STOP '  Invalid authorization  '
            ENDIF
      ENDIF
      IF(RESET .EQ. 0)THEN
*
*   If RESET is not set then check for privileges to be turned on
*
      	 IF(MSKON .NE. 0)THEN
*
*   If the TT field is blank then we're in batch mode.
*   Notify the operator.
*
            IF((TTNAM(1).EQ.0).AND.(TTNAM(2).EQ.0))THEN
      	         CALL OPCOM(13,'Setprv-I-Prvirqst Usr_nm > '
     -                    //NAME//'-Batch ',)
      	    ELSE
*
*  Otherwise we're interactive and notify the operator.
*
                 CALL OPCOM(13,'Setprv-I-Prvirqst Usr_nm > '
     -                    //NAME,)
      	    ENDIF
*
*   Notify him of the privileges requested, the reason, and
*   if he requires an operator aknowlegement pass that
*   on to ""opcom"" to take care of.
*
            CALL OPCOM(13,'Prv_rqstd > '//PRIV,)
      	    IF(OPRPLY.EQ.0)THEN
                 CALL OPCOM(13,'Reason for request > '//REASON,)
      	    ELSE
                 CALL OPCOM(13,'Reason for request > '//REASON,DUMMY)
      	    ENDIF
      	 ENDIF
      ENDIF
*
*    Set up the remainder of the arguments to SYS$SETPRV
*
*     ARGLST(1) Contains the number of arguments (4)
*               to be passed to SYS$SETPRV.
*     ARGLST(2) Contains the flag to indicate the privileges
*               should be set rather than disabled. (1)
*     ARGLST(3) Contains the actual mask of privileges that are
*               being set. (variable based on PRIV and MSK)
*     ARGLST(4) Contains the flag to indicate the permanance
*               of the privileges being set.   (1)
*     ARGLST(5) Contains a flag to indicate that the previous 
*               privilege mask is not to be returned. (0)
*
*
*     Issue the CMEXEC system service passing the name
*     SYS$SETPRV as the routine to be called once the
*     mode has been changed to executive. The requirement
*     is to pass the argument list contained in ARGLST to 
*     SYS$SETPRV and pass the completion status in ISTAT.
*     Care should be excersized here as an improper call
*     to the change mode instruction can cause the system
*     to crash.
*
      ARGLST(1)=4
      ARGLST(3)=%LOC(MSK)
      ARGLST(4)=1
      ARGLST(5)=0
 20   IF(ADD.EQ.0)THEN
*
*  If /ADD has not been specified then set him back to the authorized mask
*   before giving him what he has requested.
*
      	MSK=MSKAUTH
        ARGLST(2)=1
        ISTAT=SYS$CMEXEC(%VAL(%LOC(SYS$SETPRV)),ARGLST)
         IF(ISTAT.NE.1)THEN
            TYPE '(1X,8HERRSTAT=,Z8)',ISTAT
            STOP 'Error image not installed or insf priv'
         ENDIF
*
*     Check the status return on SYS$CMEXEC.
*     If not a normal status return print an error
*        message and quit.
*
      ENDIF 
      IF(RESET.EQ.1)THEN
*
* If reset has been specified then set him back to authorized level.
* Also check for a bad return and print out the error status.
*
      	 MSK=.NOT.MSKAUTH 
         ARGLST(2)=0
         ISTAT=SYS$CMEXEC(%VAL(%LOC(SYS$SETPRV)),ARGLST)
         IF(ISTAT.NE.1)THEN
            TYPE '(1X,8HERRSTAT=,Z8)',ISTAT
            STOP 'Error image not installed or insf priv'
         ENDIF
      ELSE
*
* Otherwise the privileges must be set. Determine the contents
*   of MSKON and MSKOFF assume that /ADD is on and additively
*   set the privileges he has requested to what he already has.
*  Don't forget to check for a bad status return.
*
      	 MSK=MSKON
         ARGLST(2)=1
         ISTAT=SYS$CMEXEC(%VAL(%LOC(SYS$SETPRV)),ARGLST)
         IF(ISTAT.NE.1)THEN
            TYPE '(1X,8HERRSTAT=,Z8)',ISTAT
            STOP 'Error image not installed or insf priv'
         ENDIF
*
*   We are here based on the assumption that these privileges are 
*    to be set additively to the authorized mask. Also we will
*    turn off bits that have been set in MSKOFF.
*
      	 MSK=.NOT.(MSKON .OR. MSKAUTH) .OR. MSKOFF
         IF (ADD.EQ.1)MSK=MSKOFF
         ARGLST(2)=0
         ISTAT=SYS$CMEXEC(%VAL(%LOC(SYS$SETPRV)),ARGLST)
         IF(ISTAT.NE.1)THEN
            TYPE '(1X,8HERRSTAT=,Z8)',ISTAT
            STOP 'Error image not installed or insf priv'
         ENDIF
      ENDIF
*
* If none of the above condidtions has been met, then print out some
*  help about the format of the standard command.
*
      IF((RESET.EQ.0).AND.(MSKON.EQ.0).AND.(MSKOFF.EQ.0).AND.(ADD.EQ.0))
     - THEN 
      WRITE(6,119)
  119 FORMAT(' Command line format for SETP is',/
     -,'   SETP [/ADD] [/RESET] [/PRIV=(list of privs) "Reason',//)
      WRITE(6,129)
      WRITE(6,139)'ACNT    ','ALLSPOOL','ALTPRI  ','BUGCHK  '
     -        ,'BYPASS  ','CMEXEC  ','CMKRNL  ','DETACH  '
     -        ,'DIAGNOSE','EXQUOTA ','GROUP   ','GRPNAM  '
     -        ,'LOG_IO  ','MOUNT   ','NETMBX  ','OPER    '
     -        ,'PFNMAP  ','PHY_IO  ','PRMCEB  ','PRMGBL  '
     -        ,'PRMMBX  ','PSWAPM  ','SHMEM   ','SYSGBL  '
     -        ,'SYSLCK  ','SYSNAM  ','SYSPRV  ','TMPMBX  '
     -        ,'VOLPRO  ','WORLD   '
  129 FORMAT('  The following is a list of available privileges:',/)
  139 FORMAT(6(4X,A8),/)
      ENDIF
*
*     The STOP statement was purposely left out here to more
*     closely emulate the appearance of a normal DEC standard image.
*
      END
      SUBROUTINE PARSE(REASON,RQST,MSKON,MSKOFF,LEVEL,ADD,RESET)
*
* PARSES COMMAND LINE FOR SETPRV
*
* This routine will pass back the basic contents of the command line.
* It will also call subroutine Build_msk to convert the alpha numeric
* representation of the privileges to a binary field in the form required 
* by the system service SETPRV.
*
*  Called by MAINLINE
*
*  Calls BUILD_MSK
*        CLI$ Routines to determine the presence or absence of a qualifier
*             and the value if it requires a value specified.
*
      EXTERNAL CLI$PRESENT
      INTEGER*4 ADD,RESET,MSKON,MSKOFF,LEVEL
      CHARACTER*132 RQST,REASON,REQPRV*10
      INTEGER*4 ERR,DUMLVL,CLI$PRESENT
*
*  Initialize variables 
*

      MSKON=0
      MSKOFF=0
      ADD=0
      RESET=0
      I=0
      NON=1
      IEND=0
*
*  Is /ADD present if so then set the switch to one to be passed to the 
*  mainline.
*
      IF(CLI$PRESENT('ADD'))ADD=1
*
*  If reset is on then add cant be on. 
*
      IF(CLI$PRESENT('RESET'))THEN
           RESET=1
      	   ADD=0
*
*  Otherwise /RESET isnt on proceed to get the values of the
*   privileges and any that need to be turned on or off.
*
      ELSE
*
*  As long as it's not the end keep getting them one at a time.
*
        DO WHILE(IEND.EQ.0)
            CALL CLI$GET_VALUE('PRIV',REQPRV)
            IF(REQPRV.EQ.' ')THEN
              IEND=1
      	    ELSE
*
*  Build the mask iteratively one privilege at a time
*         based on "ON" or "OFF"
*
              IF(REQPRV(1:2).EQ.'NO')THEN
      	        CALL BUILD_MSK(REQPRV(3:10),MSKOFF,ERR,DUMLVL)	
       	      ELSE
      	        CALL BUILD_MSK(REQPRV(1:8),MSKON,ERR,LEVEL)	
*
*   As long as there is no error from build_mask keep it up
*
*    The error might be an ambigous refrence (you abbreviated to short)
*     or an illegal privilege based on the level authorized by the 
*     system manager.
*
      		IF(ERR.EQ.0)THEN
      	          IF(NON.NE.1) THEN
      	  	    RQST=RQST(1:NON)//','
       		    NON=NON+1
      		  ENDIF
                  RQST=RQST(1:NON)//REQPRV(1:8)
                  NON=NON+8
      	          DO WHILE (RQST(NON:NON).EQ.' ')
      		    NON=NON-1
      	          ENDDO
                ELSE
*
*   If it's an error determine why and print an error
*    there may be enough left to salvage though 
*     so keep going based on that assumption.
*
                  IF(ERR.EQ.1)WRITE(6,19)REQPRV
                  IF(ERR.EQ.2)WRITE(6,29)REQPRV
      	        ENDIF
              ENDIF
      	    ENDIF
        ENDDO
*
*   Get the reason from the command line if it's blank
*   prompt for it.
*
        CALL CLI$GET_VALUE('REAS',REASON)
      	DO WHILE(REASON(1:4) .EQ. '    ')
      	    WRITE(6,'(A,$)')' Reason required > '
      	    READ(5,456)REASON
 456        FORMAT(A)
      	ENDDO
      ENDIF
*
*   You mean to tell me I'm finally done
*
      RETURN
   19 FORMAT(1X,' Illegal Privilege  ',A)
   29 FORMAT(1X,'Ambiguous Reference ',A)
      END      
      SUBROUTINE VALIDATE(PRIV,UNAME,MSKON,LEVEL,TTDEV,TTCNTRL,
     -TTPORT,TODAY,TIME,OPRPLY,IERR,ICLASS)
*
*
*   Routine Name VALIDATE
*
*   Author : SSgt. Michael D. Weaver 
*
*            We would like to acknowlege the very significant
*            contributions of Dave Lennig (DEC software resident)
*            in the conceptual and initial development of this
*            system.
*
*   Date : 15 Feb 1983
*
*
*  Record definition
*
*   field 1    CODE integer*4 key for indexed access
*   field 2    NAME last name of the valid user CHARACTER*20
*   field 3    CLASS classification of PRIVILEGES authorized
*   field 4    DEVICE TT ports where the privileges can be used
*   field 5    BEFORE privilege can't be used before this time
*   field 6    AFTER  time after which the privileges can't be used
*   field 7    DAYCODE code for weekday or weekend usage
*   field 8    PRVLST privivege list for classification .eq. 0
*
      IMPLICIT INTEGER (A-Z)
      CHARACTER REQPRV*10,PRIV*132,UNAME*20,BUFF*30
      CHARACTER NAME*20,PRVLST*60
      CHARACTER TERM*11/'SYS$COMMAND'/
      INTEGER*4 DONE,CODEOUT,IADD,LVL,IERR,MSKON,NON,ERR
      INTEGER*4 SYS$QIOW,TTCHAN,I_QIOSTAT,FUNC,TABLE(16)
      INTEGER*2 CLASS,BEFORE,AFTER,DAYCODE,OPERACK,ICLASS,BTCHNET
      INTEGER*2 TTDEV,TTPORT,TTCNTRL,TODAY,TIME,OPRPLY
      BYTE RECORD(128),DEVICE(8),BUFF1(30)
      EQUIVALENCE (RECORD(1),CODEOUT),(RECORD(5),NAME)
     -            ,(RECORD(25),CLASS),(RECORD(27),DAYCODE)
     -	,(RECORD(29),BEFORE),(RECORD(31),AFTER),(RECORD(33),DEVICE) 
     -	,(RECORD(41),OPERACK),(RECORD(43),BTCHNET),(RECORD(46),PRVLST)
      EQUIVALENCE (BUFF,BUFF1)
*
*  Get the definitions for OPCOM and the QIOW stuff
*
      INCLUDE '($IODEF)'
      INCLUDE '($SSDEF)'
*
*   Open the data file 
*
      OPEN(8,ACCESS='KEYED',CARRIAGECONTROL='NONE',ERR=999
*
*   Youll have to change this line or define a LOGICAL for SYS$PROCEDURE:
*
     -    ,FILE='SYS$PROCEDURE:PRVAUTH'
     -    ,KEY=(1:4:INTEGER,5:24:CHARACTER)
     -    ,ORGANIZATION='INDEXED',RECL=128,STATUS='OLD'
     -    ,READONLY,SHARED)
      IOS=0
*
*    Setup a loop to read the the authorization code and encrypt it
*  if an error is encountered RETURN if not try and read the data
*  file and check if there is a matching entry in the data file.
*  If there is a matching entry in the data file return with an error
*  otherwise this is a valid request and return with a status of DONE.
*
      DONE=0
      NTRIES=1
*
*    Set up a table of seed values for the polynomial cyclic 
*   redundancy check. This is used in the upcoming call to LIB$CRC
*
      CALL LIB$CRC_TABLE('EDB88320'X,TABLE)
*
*   Assign a channel to the terminal for the QIOW
*   and incorporate the required functions
*     
      CALL SYS$ASSIGN(TERM,TTCHAN,,)
*
*       IO$_READVBLK  : Read virtual block
*       IO$M_NOECHO   : Do a noecho read from the terminal
*       IO$M_TIMED    : Time the read based on one of the parameters
*                       to the QIOW
*
      FUNC= IO$_READVBLK .OR. IO$M_NOECHO .OR. IO$M_TIMED
*
*   Set up a loop to give the guy three tries at getting a good password
*        and get out if you find a good password
*
      DO WHILE((DONE.EQ.0).AND.(NTRIES.LE.3))
*
*  Before the third try keep setting this to 0
*
         IERR=0
*
*   Increment the NTRIES for each try
*
         NTRIES=NTRIES+1
         WRITE(6,'(1X,A,$)')' Enter authorization-> '
*
* If its not a batch job "TTDEV=0" setup and read it from
*  the terminal.
*
         IF(TTDEV.NE.0)THEN
*
*  Clear the buffer and do the read using the channel
*  defined earlier, use the 10 as the timer value and the
*  "FUNC" defined earlier for noecho and such. The try at the password
*  will be placed in BUFF
*  
            BUFF(1:30)='                              '
            I_QIOSTAT=SYS$QIOW(,%VAL(TTCHAN),%VAL(FUNC),,,,
     -           %REF(BUFF),%VAL(LEN(BUFF)),%VAL(10),,,)
            IF(.NOT. I_QIOSTAT)CALL LIB$STOP(%VAL(I_QIOSTAT))
         ELSE
*
*  Otherwise it's a batch job do a normal FORTRAN read
*
            NTRIES=4
            READ(5,119,ERR=123)NB,BUFF
*
*  Put a carriage return at the end of the password just in case it's
*  BATCH
*
      	    BUFF(NB+1:NB+1)=CHAR(13)
  119	FORMAT(Q,A)
         ENDIF
*
*  In any event convert the resulting password to uppercase
*
         ISTAT=STR$UPCASE(BUFF,BUFF)
*
*  Do the CRC against the attempt at a good password
*  There is an exponential chance that there will be a duplicate
*  But it is non-returnable. I used the technique described in the 
*  VAX architecture handbook.
*
*  It corresponds to the AUTODIN II algorithm
*  
*  The LOGICAL .NOT. is required by the algorithm
*
         CODEOUT=LIB$CRC(TABLE,-1,%DESCR(BUFF))
         CODEOUT=.NOT. CODEOUT
*
*   Try and read the record from the data file using the 
* encrypted authorization code as a key.
*
         READ(8,KEY=CODEOUT,KEYID=0,IOSTAT=IOS,ERR=998)RECORD
*
*  If successful set DONE to escape the loop so the classification
* can be determined and the proper privileges, etc. can be done.
* 
         DONE=1
 998     ICLASS=CLASS
*
*   If TTDEV is 0 then it's a BATCH job
*
         IF(TTDEV.EQ.0)THEN
*
*   Determine from the contents of BTCHNET if he is allowed to use 
*   privileges batch if not (IF((BTCHNET.AND.1).EQ.0) then 
*   pretend that the read was invalid and set IOS=188
*
*   Before you leave this part of the routine print an error message
*
            IF((BTCHNET.AND.1).EQ.0)THEN
                IOS=188
      	    ENDIF
         ENDIF
         IF(TTDEV.EQ.3)THEN
*
*   Determine from the contents of BTCHNET if he is allowed to use 
*   privileges on a remote "DECNET" if not (IF((BTCHNET.AND.2).EQ.0) then 
*   pretend that the read was invalid and set IOS=199
*
*   Before you leave this part of the routine print an error message
*
            IF((BTCHNET.AND.2).EQ.0)THEN
                IOS=199
      	    ENDIF
         ENDIF
         IF(IOS .NE. 0)THEN
* 
*  If there is no successful read from the data file then 
* no record exists for the encrypted authorization code.
* Print an error message and RETURN.
*
 123     IERR=1
         WRITE(6,'(A)')'$Invalid authorization '
         IF(IOS.EQ.188)WRITE(6,'(A)')'$Not authorized Batch execution'
         IF(IOS.EQ.199)WRITE(6,'(A)')'$Not authorized Remote execution'
        ENDIF
      ENDDO
*
*  If there are no errors so far check the terminal port
*
      IF(IERR.EQ.0)THEN
           IF(TTDEV.EQ.1)THEN
       	        IF((DEVICE(TTCNTRL).AND.TTPORT).EQ.0) THEN
       		     IERR=2
      		     WRITE(6,'(A)')'$Illegal terminal port '
      	        ENDIF
           ENDIF
*
*  Then check the day and determine if it's a legal day
*
           IF((TODAY.AND.DAYCODE).EQ.0)THEN
      	        IERR=3
      	        WRITE(6,'(A)')'$Illegal day '
           ENDIF
*
*   Check the time is it legal
*
           IF(BEFORE.LT.AFTER)THEN
              IF((TIME.LT.BEFORE).OR.(TIME.GT.AFTER))THEN
                   IERR=4
      	           WRITE(6,'(A)')'$Illegal time '
              ENDIF
      	   ELSE
      	      IF((TIME.GT.AFTER).AND.(TIME.LT.BEFORE))THEN
                   IERR=4
      	           WRITE(6,'(A)')'$Illegal time '
      	      ENDIF
           ENDIF
      ENDIF
*   
*  Now its time to check the magical operator acknowlegement
*  flag it can be one of four values.
*    
      IF(IERR.EQ.0)OPRPLY=OPERACK.AND.1
      IF((IERR.GT.1).AND.(OPERACK.GE.2))THEN
      	   IERR=0
           OPRPLY=1
      ENDIF
      IF(IERR.GT.0)RETURN
      IF(DONE .NE. 1) RETURN
      UNAME=NAME
*
*    Based on classification and a valid request give the user "GROUP"
* if the classification is "1". If the classification is "0" then give
* the user the privilege list exactly as it appears in his authorization
* record, accept the reason and return.
*
      IF(CLASS .LE. 1)THEN
      	PRIV='GROUP '
*
*  Check the levels and privileges if it's above level 1
*  and build the alphanumeric mask.
*
        IF(CLASS.EQ.0) PRIV=PRVLST
        MSKON=0
        NON=1
        DONE=0
        DO WHILE(DONE.EQ.0)
          REQPRV=PRIV(NON:NON+7)
          J=0
          DO I=1,10
            IF(((REQPRV(I:I).EQ.' ').OR.(REQPRV(I:I).EQ.','))
     -          .AND.(J.EQ.0))J=I
            IF(J.NE.0)REQPRV(I:I)=' '
          ENDDO
          NON=NON+J
          IF((REQPRV.EQ.' ').OR.(J.EQ.0))THEN
*
*  We're here based on the assumption that were still at an authorized
*   level and that the command line has not been completely parsed
*   build the privilege list iteratively
*
            DONE=1
      	  ELSE
      	      CALL BUILD_MSK(REQPRV(1:8),MSKON,ERR,LVL)	
      	  ENDIF
        ENDDO
      ELSE
*
*  Check to determine if this particular privilege is above
*  the authorized level if so quit.
*
        IF(CLASS.LT.LEVEL)THEN
          IERR=1
          WRITE(6,19)
        ENDIF
      ENDIF
      RETURN
   19 FORMAT(1X,' REQUESTED PRIVILEGE ABOVE AUTHORIZED LEVEL')
*
*   If an error is encountered opening the data file come
* here and stop.
*
999   STOP 'Error opening prvauth.dat'
      END
      SUBROUTINE OPCOM(OPRNO,MSG,MBXNUM)
*
*    Subroutine : OPCOM
*
*    Author : SSgt. Michael D. Weaver 
*
*    Date : 15 Feb 1983
*
*    	Again I would like to acknowlege the contributions
*     of Mr. Dave Lennig in the development of this subroutine.
*
*    Variables : 
*     
*       MBXNUM : An integer variable passed from the caller.
*       Normally this will be a null argument as the program is 
*       implemented. There is a facility for passing a mailbox
*       number to allow an operator to reply to the request for
*       privileges.
*
*       MSG    : A character variable of 'passed' length. This 
*       variable is used to hold the actual message text to be sent
*       to """OPCOM""".
*      
*
*       OPCBUF : A character variable of length 128. It is used to 
*       the major portion of the argument list to the system service
*       SNDOPR. It is equivalenced to the minor fields of the argument
*       buffer.
*
*       OPER   : This is an integer array variable of 16 bits. It is used 
*       for holding an array of the possible operator types from which
*       a selection can be made as to which operator the request is to
*       be sent.
*
*       OPRNO  : An integer variable of 32 bits. Passed from the 
*       calling routine. Allows the calling routine to select the
*       operator which will recieve the message.
*
*       TARGET : Used to determine from the possible operator types as
*       defined in the $OPCDEF definition file what operator is 
*       to recieve notification of requests for privileges in addition 
*       to the console and the "operator.log" file. Currently I have 
*       included all possible operator types to allow for future expansion.
*       
*       
*    
      INCLUDE '($OPCDEF)'
      INCLUDE '($IODEF)'
      INTEGER OPER(13)/OPC$M_NM_OPER1,OPC$M_NM_OPER2,OPC$M_NM_OPER3
     1,OPC$M_NM_OPER4,OPC$M_NM_OPER5,OPC$M_NM_OPER6,OPC$M_NM_OPER7
     2,OPC$M_NM_OPER8,OPC$M_NM_OPER9,OPC$M_NM_OPER10,OPC$M_NM_OPER11
     3,OPC$M_NM_OPER12,OPC$M_NM_CENTRL/
      INTEGER*4 OPRNO,TARGET,MSGID,FUNC
      INTEGER*2 OPC$_RQSTABORT/'801C'X/
      INTEGER*2 OPC$_RQSTCMPLTE/'8029'X/
      INTEGER*2 OPC$_NOPERATOR/'8061'X/
      INTEGER*2 OPC$_RQSTPEND/'8021'X/
      INTEGER*2 OPC$_RQSTCAN/'8084'X/
      INTEGER SYS$CREMBX
      INTEGER*2 IOSB(4),STAT(20)
      BYTE RQST/OPC$_RQ_RQST/
      CHARACTER*128 OPCBUF
      CHARACTER MBXNAM*9 /'MSG_REPLY'/
      CHARACTER*(*) MSG
*
*     This equivalence sets up the buffer that SNDOPR will
*     require to send a message to """OPCOM""" (the system 
*     support routine).
* 
*     OPCBUF(1:) Will contain the function requested from
*     SYS$SNDOPR. This function can be changed to allow the
*     system operator to authorize requests for privileges in
*     much the same manner as a tape mount is handled. If this
*     is done please remember that all calls to this routine must
*     have a nonzero argument MBXNUM. Currently all calls specify a
*     null argument which specifies that no REPLY/TO is required
*     from the operator.
*
*     OPCBUF(2:) Will contain the target of the message which
*     can be any one of the possible OPER types as defined by
*     the DCL command REPLY/ENABLE=(kywd,...). Currently the
*     operator type is hardwired to be CENTRAL. 
*
*     OPCBUF(3:) Will contain the mailbox number used by 
*     """OPCOM""" and this routine to communicate the message
*     and any associated errors.
*
      EQUIVALENCE(OPCBUF(1:),RQST),(OPCBUF(2:),TARGET)
     -,(OPCBUF(5:),MSGID)
      FUNC= IO$_READVBLK 
*
*     This assignment sets up which type(s) of operators the
*     message will go to.
*
      TARGET=OPER(OPRNO)
      MSGID=0
*
*     This IF verifies the caller specified a null argument
*     in the mailbox parameter.
*
      IF(%LOC(MBXNUM).NE.0)THEN
         ISTAT= SYS$CREMBX(,MBXNUM,%VAL(40),,,,MBXNAM)
         MSGID=MBXNUM
      ENDIF
      
*
*     Put the message text in the argument buffer.
*
      OPCBUF(9:)=MSG
      IF(%LOC(MBXNUM).NE.0)THEN
         CALL SYS$SNDOPR(OPCBUF(1:LEN(MSG)+8),%VAL(MBXNUM))
*
*   Here is where we wait to see if the operator grants denies or pends
*   the request.
*
 77      CALL SYS$QIOW(,%VAL(MBXNUM),%VAL(FUNC),IOSB,,,
     -           %REF(STAT),%VAL(40),,,,)
         CALL SYS$WAITFR(%VAL(0))
*
*   Give the appropriate feedback if operator acknowlegement is required
*
         IF(STAT(2) .EQ. OPC$_RQSTCMPLTE)THEN
      		RETURN
         ELSEIF(STAT(2) .EQ. OPC$_RQSTABORT)THEN
                STOP 'Request denied'
         ELSEIF(STAT(2) .EQ. OPC$_NOPERATOR)THEN
                STOP 'No operator available'
         ELSEIF(STAT(2) .EQ. OPC$_RQSTPEND)THEN
                WRITE(6,'(A)')' Request pending'
      		GOTO 77
         ELSE
                GOTO 77
      	 ENDIF
      ELSE
*
*    This is the simple call where no operator interaction is required
*
         CALL SYS$SNDOPR(OPCBUF(1:LEN(MSG)+8),MBXNUM)
      ENDIF
      RETURN
      END
      SUBROUTINE BUILD_MSK(PRIV,MSK,IERROR,ILVL)
*
*
*
*     Subroutine BUILD_MSK
*
*     Author SSgt Michael D. Weaver
*
*     Date  15 Feb 1983
*
*
      INTEGER*4 MSK,IERROR,ILVL
      INTEGER*2 FOUND,DONE
      INTEGER*2 MIN(30),MAX(30),LEVEL(30),BIT(30)
      CHARACTER*8 PRV(30),PRIV
      EXTERNAL PRV$V_ACNT,PRV$V_ALLSPOOL,PRV$V_SETPRI,PRV$V_BUGCHK
      EXTERNAL PRV$V_BYPASS,PRV$V_CMEXEC,PRV$V_CMKRNL,PRV$V_DETACH
      EXTERNAL PRV$V_DIAGNOSE,PRV$V_EXQUOTA,PRV$V_GROUP,PRV$V_GRPNAM
      EXTERNAL PRV$V_LOG_IO,PRV$V_MOUNT,PRV$V_NETMBX,PRV$V_OPER 
      EXTERNAL PRV$V_PFNMAP,PRV$V_PHY_IO,PRV$V_PRMCEB,PRV$V_PRMGBL
      EXTERNAL PRV$V_PRMMBX,PRV$V_PSWAPM,PRV$V_SHMEM,PRV$V_SYSGBL
      EXTERNAL PRV$V_SYSLCK,PRV$V_SYSNAM,PRV$V_SYSPRV,PRV$V_TMPMBX
      EXTERNAL PRV$V_VOLPRO,PRV$V_WORLD
*
*   define all the possible alpha privileges
*
      DATA PRV/'ACNT    ','ALLSPOOL','ALTPRI  ','BUGCHK  '
     -        ,'BYPASS  ','CMEXEC  ','CMKRNL  ','DETACH  '
     -        ,'DIAGNOSE','EXQUOTA ','GROUP   ','GRPNAM  '
     -        ,'LOG_IO  ','MOUNT   ','NETMBX  ','OPER    '
     -        ,'PFNMAP  ','PHY_IO  ','PRMCEB  ','PRMGBL  '
     -        ,'PRMMBX  ','PSWAPM  ','SHMEM   ','SYSGBL  '
     -        ,'SYSLCK  ','SYSNAM  ','SYSPRV  ','TMPMBX  '
     -        ,'VOLPRO  ','WORLD   '/
*
*   Define the minimum number of characters that can be used
*   to abbreviate a given privilege
*
      DATA MIN/2,3,3,2,2,3,3,3*2,3,3,6*2,3*4,2,2,4*4,3*2/
*
*  All the maximums for the numbers of characters in the privileges
*
      DATA MAX/4,8,6*6,8,7,5,6,6,5,6,4,6*6,5,6*6,5/
*
*  The levels for all the privileges as contained in the SMOG 
*
      DATA LEVEL/2,2,3,2,4*5,4,2,1,2,5,1,1,3,5,5,3*2,3,2
     -          ,4,3,5,5,1,4,3/
      FOUND=0
      IERROR=0
      J=0
*
* Until you run through all of the thirty possible privileges
* or until you find that this is a match, search for the privilege
* and determine if its listed, is a valid abbreviation, and
* pass back the level of the highest privilege requested so that
* ROUTINE "VERIFY" can check if this guy is authorized a privilege
* at this level.
*
      DO WHILE ((J.LE.30) .AND. (FOUND.EQ.0))
      	J=J+1
      	I=0
        DONE=0
      	DO WHILE (DONE.EQ.0)
      	  I=I+1
      	  IF(PRV(J)(I:I).NE.PRIV(I:I))THEN
      	    DONE=1
      	    IF(PRIV(I:I).EQ.' ')THEN
      	      IF(MIN(J).LT.I) THEN
      	        FOUND=1
      		PRIV=PRV(J)
              ELSE
      		IERROR=2
      	      ENDIF
      	    ELSE
      	      IF(MAX(J).LT.I) FOUND=1
      	    ENDIF
      	  ENDIF
      	ENDDO
      ENDDO
      IF(FOUND.EQ.1) THEN
*
*  This is where you determine what the highest level of privilege
*  that has been requested.
*
      	IF(ILVL.LT.LEVEL(J))ILVL=LEVEL(J)
*
* Here is where I actually set the bits
*
        IF(J.EQ.1)MSK=JIBSET(MSK,%LOC(PRV$V_ACNT))
        IF(J.EQ.2)MSK=JIBSET(MSK,%LOC(PRV$V_ALLSPOOL))
        IF(J.EQ.3)MSK=JIBSET(MSK,%LOC(PRV$V_SETPRI))
        IF(J.EQ.4)MSK=JIBSET(MSK,%LOC(PRV$V_BUGCHK))
        IF(J.EQ.5)MSK=JIBSET(MSK,%LOC(PRV$V_BYPASS))
        IF(J.EQ.6)MSK=JIBSET(MSK,%LOC(PRV$V_CMEXEC))
        IF(J.EQ.7)MSK=JIBSET(MSK,%LOC(PRV$V_CMKRNL))
        IF(J.EQ.8)MSK=JIBSET(MSK,%LOC(PRV$V_DETACH))
        IF(J.EQ.9)MSK=JIBSET(MSK,%LOC(PRV$V_DIAGNOSE))
        IF(J.EQ.10)MSK=JIBSET(MSK,%LOC(PRV$V_EXQUOTA))
        IF(J.EQ.11)MSK=JIBSET(MSK,%LOC(PRV$V_GROUP))
        IF(J.EQ.12)MSK=JIBSET(MSK,%LOC(PRV$V_GRPNAM))
        IF(J.EQ.13)MSK=JIBSET(MSK,%LOC(PRV$V_LOG_IO))
        IF(J.EQ.14)MSK=JIBSET(MSK,%LOC(PRV$V_MOUNT))
        IF(J.EQ.15)MSK=JIBSET(MSK,%LOC(PRV$V_NETMBX))
        IF(J.EQ.16)MSK=JIBSET(MSK,%LOC(PRV$V_OPER  ))
        IF(J.EQ.17)MSK=JIBSET(MSK,%LOC(PRV$V_PFNMAP))
        IF(J.EQ.18)MSK=JIBSET(MSK,%LOC(PRV$V_PHY_IO))
        IF(J.EQ.19)MSK=JIBSET(MSK,%LOC(PRV$V_PRMCEB))
        IF(J.EQ.20)MSK=JIBSET(MSK,%LOC(PRV$V_PRMGBL))
        IF(J.EQ.21)MSK=JIBSET(MSK,%LOC(PRV$V_PRMMBX))
        IF(J.EQ.22)MSK=JIBSET(MSK,%LOC(PRV$V_PSWAPM))
        IF(J.EQ.23)MSK=JIBSET(MSK,%LOC(PRV$V_SHMEM))
        IF(J.EQ.24)MSK=JIBSET(MSK,%LOC(PRV$V_SYSGBL))
        IF(J.EQ.25)MSK=JIBSET(MSK,%LOC(PRV$V_SYSLCK))
        IF(J.EQ.26)MSK=JIBSET(MSK,%LOC(PRV$V_SYSNAM))
        IF(J.EQ.27)MSK=JIBSET(MSK,%LOC(PRV$V_SYSPRV))
        IF(J.EQ.28)MSK=JIBSET(MSK,%LOC(PRV$V_TMPMBX))
        IF(J.EQ.29)MSK=JIBSET(MSK,%LOC(PRV$V_VOLPRO))
        IF(J.EQ.30)MSK=JIBSET(MSK,%LOC(PRV$V_WORLD ))
      ELSE
      	IF(IERROR.EQ.0)IERROR=1
      ENDIF
      RETURN
      END
