	PROGRAM USERPRINT
C
C	VERSION 1.9	20-Oct-1983
C
C	Written by:	Jim Moomaw
C			INTEL CORPORATION
C			M.S. AL3-1-255
C			3585 S.W. 198th Ave
C			Aloha, OR  97007
C
C			Reliability Physics Laboratory,  Dept. 255
C
C
C---------DISCUSSION OF EXTERNALS.
C
C This is a program to provide custom printing services not available 
C with the VMS DCL-level PRINT command, offering pitch/margin/letter-
C quality printing options (for LA100's, LA120's, related devices),
C automatic queue selection according to terminal requesting printing,
C optimum horizontal pitch selection (by approximation), and access to
C files located on allocated devices. It contains quite a bit of feed-
C back to the user as to what's going on, error messages when things
C go wrong, flags invalid syntax, and in most cases tries to do as much
C as it can despite errors. 
C
C Specifically, it provides parsing of a DCL-level PRINT command with 
C a custom Command Language Descriptor file which mimics, but replaces
C PRINT parsing as provided by DCLTABLES. The program builds a new print
C command procedure file, copies print files to a logical system direc-
C tory (called SYS$SYSPRINT:) appending appropriate escape sequences at
C front and back of files, and calld a command procedure to execute the
C print job using the original VMS DCL PRINT for actual queueing. 
C
C This alternate PRINT command was generated to provide new qualifiers
C for our various types of printing devices (for simplifying the user
C interface), and to provide automatic routing of print jobs to the
C printer in the same building as the user, since our system is linked
C to several with terminals and printers in each. It also allows for the
C screening out of some obviously non-printable jobs, but can't do a 
C rigorous job of this due to the lack of unambiguous file indicators
C telling what's printable, and what isn't.
C
C This program works in concert with several other procedures and files
C to pull all of this off.  The procedural sequence is as follows:
C
C o A Command Description File, called USERPRINT.CLD, is installed by
C   the system manager in SYS$LIBRARY. It contains all of the syntax
C   desired by the new PRINT command (where most of the syntax mimics
C   the original DCLTABLES version of PRINT.)
C
C o The Command Definition Utility (SET COMMAND) is run during user-
C   login time to install USERPRINT.CLD parsing tables into the user's
C   process (to override the default DCL PRINT command.) (Modifying
C   the DCLTABLES wasn't done, since the DCL PRINT command is ultimately
C   used to queue the files.)
C
C o USERPRINT.CLD has identified this program USERPRINT.EXE to be run
C   whenever a user types a PRINT command to the DCL dollar-sign prompt.
C   This program parses out the input line, looks to see what terminal 
C   the user is at, looks at the types of files listed to be printed,
C   looks at the qualifiers given, looks at the device specified for
C   printing, copies a modified form of legal files to a directory on
C   the system disk, builds a new PRINT command in regular DCLTABLES
C   syntax, builds a command procedure containing the actual print
C   command, and invokes a command procedure USERPRINT.COM to commence
C   the printing sequence.
C
C o USERPRINT.COM installs the VMS DCLTABLES so that it can use normal
C   PRINT commands, and then passes the command procedure made by
C   USERPRINT.EXE to DCL for print queueing.  USERPRINT.CLD is then
C   re-installed with SET COMMAND to again enable special PRINT command
C   parsing.  The files are printed with the delete qualifier, so are
C   deleted from the system print directory after printing.
C
C
C The  program  handles  the  parameters  and  qualifiers  defined  in
C USERPRINT.CLD as follows:
C
C o Parameters
C
C   FILESPEC     - Inspected to see if it exists, and if it is 
C                  formatted. If both true, file(s) are copied over
C                  to SYS$SYSPRINT:, with appropriate escape
C                  sequences appended on the front and back of the
C                  file as needed, to implement the qualifiers
C                  specified in the command line. A new PRINT command
C                  line is built, using appropriate disk, directory,
C                  filename, type, and version numbers.
C
C o Qualifiers
C
C   In general, the global commands parsed out will be attached to the
C   PRINT verb in the new PRINT construct. Positional commands will be 
C   attached as originally input. Where default values are implied or
C   defined, they will be ignored and not inserted into the new PRINT
C   line construct.
C
C   Qualifier   Placement    Action Taken
C   ____________________________________________________________________
C   AFTER       GLOBAL       As defined in the DCL help command. Not
C                            allowed for SYS$TERMINAL queue.
C
C   COPIES      POSITIONAL   As defined in the DCL help command, with
C                            the exception that maximum number of copies
C                            will be limited to 10
C
C   DEVICE      GLOBAL       Takes on a value, indicating which printer
C                            device should get the job. Legal choices 
C                            are: AL3LA100, JF1LA100, VERSATEC, LA120,
C                            TTD1, TTD3, SYS$PRINT, SYS$TERMINAL
C
C   DELETE	POSITIONAL   As defined in the DCL help command, with
C                            the restriction that it can only be used
C                            with UNFORMATTED files (since FORMATTED 
C			     files are physically different than the
C			     source files, but UNFORMATTED files are
C                            physically same.)
C
C   DRAFT       POSITIONAL   If selected, indicates draft-quality mode.
C                            Mutually exclusive with LETTER qualifier.
C                            Only valid for LA100 type printers, and
C                            ignored for other printer types.
C
C   FLAG_PAGE   POSITIONAL   As defined in the DCL help command.
C
C   FONT        POSITIONAL   Takes on a value, indicating which font 
C                            type to use. Legal choices are: COURIER10,
C                            ORATOR10, GOTHIC12, UPPER, and LOWER.
C                            Vertical pitch is set to 6lpi for all fonts
C                            except orator, which is set to 4lpi.
C                            Top & Bottom margins are set to 1&64 for all
C                            fonts except orator, which is set to 1&40.
C                            This qualifier is only valid for LA100-type
C                            printers, and ignored for other non-font-
C                            supporting printer types. (The selection
C			     is based on assumed positions of the fonts
C                            in the printers, which are DEC's defaults.)
C
C   FORMATTING  GLOBAL       Enables or disables the insertion of escape
C                            sequences into the output files, and also
C			     enables or  disables any formatting quali-
C                            fiers  selected.   Enabled   by   default.
C                            NOFORMATTING eliminates the delay caused by
C                            the rather slow copying process, as print
C                            files aren't copied to SYS$SYSPRINT:, but
C                            the original source files are used instead.
C
C   HEADER      POSITIONAL   As defined in the DCL help command.
C
C   JOB_COUNT   GLOBAL       As defined in the DCL help command, but 
C                            limited to a value of 10.
C
C   LETTER      POSITIONAL   If selected, indicates letter-quality mode.
C                            Mutually exclusive with DRAFT qualifier.
C                            Only valid for LA100 type printers, and
C                            ignored for other printer types.
C
C   MARGIN      POSITIONAL   Takes on a value, indicating what the left
C                            horizontal margin will be (absolute.)  On
C                            LA100 printers, the left-most edge will be 
C                            physically set for column one for a MARGIN 
C                            setting of one. On the LA120, a margin
C                            setting of one is just to the right of the
C                            binder holes. The program will attempt to
C			     optimize the left margin to be approximate-
C                            ly one inch, if defaulted, for the given
C                            horizontal pitch of each file. 
C
C   NAME        GLOBAL       As defined in the DCL help command.
C
C   NOTIFY      GLOBAL       As defined in the DCL help command.
C
C   PAGE_COUNT  LOCAL	     As defined in the DCL help command.
C
C   PITCH       POSITIONAL   Takes on a value, indicating what the hori-
C                            zontal pitch will be. Not all devices have
C                            this capability, where pitch will be 
C                            ignored.   For the case of LA100
C                            printers with the LETTER qualifier set, 
C                            the PITCH qualifier has special meaning
C                            for pitches equal to or longer than the
C                            generic font-type chosen, and at half the
C                            generic type. Legal values are: 5, 6,
C                            6.6, 8.25, 10, 12, 13.2, 16.5.
C
C   PRIORITY    GLOBAL       As defined in the DCL help command.
C
C   QUEUE       GLOBAL       As defined in the DCL help command.
C
C   SPACE       POSITIONAL   As defined in the DCL help command.
C
C
C   Some inspection is done on the type of file requested for printing,
C   so that unprintable files will usually be rejected for queueing. I
C   have determined no elegant way of determining printable files from 
C   non-printable (from exhaustive empirical testing), and am left to
C   test for the obviously illegal file types (e.g. EXE, TSK, OBJ, STB,
C   SYS, DIR, PAR, DIC, MAI, HLB, OLB, TLB, MLB, SML, PIC, PLV) with the
C   hope that file types haven't been renamed. (Of course this list 
C   isn't exhaustive, and may need to be modified on occation. File
C   types of DAT are the most prone to cause problems, as some are ASCII
C   and some are not, and one doesn't really want to preclude them from
C   being printed.)
C
C   All files printed with formatting enabled will have a printer
C   initialization prefix to set the proper printer attributes for the
C   file, as well as a restoration suffix to reset the printer to the
C   normal defaults for that device (recognizing that the output file
C   might have fouled up the default printer settings.) 
C
C
C---------DISCUSSION OF INTERNALS.
C
C   The program structure is as follows:
C
C	I. LOOK TO SEE WHAT TERMINAL THE USER IS ON.
C	    This will tell us what building he is in. It will also
C           crudely indicate user group, for the case of MPD CAD.
C	    If no determination is made, exit.
C
C       II. DETERMINE WHICH PRINTER DEVICE THE USER NEEDS.
C            This requires looking to see if there's a device or queue
C	     defined explicitly, and if not, then choosing one by user
C            location.
C
C      III. PARSE INPUT STRING FOR GLOBAL SYMBOLS.
C            Set global qualifier flags for those present. For the 
C	     special case of formatting qualifiers, if that flag is
C            set then begin building the global prefix string as you
C            go. Any errors here will cause program exit. 
C
C       IV. BUILD FILE-FORMATTING SUFFIX, AND HEAD-END OF PRINT COMMAND
C	    STRING. 
C	     With flags set from above, use device type to build 
C	     file-formatting suffix string, and selected qualifiers to
C	     begin building the command line. (A distinction is made
C	     between formatting qualifiers, which are used to build
C	     the prefix and suffix strings, and command qualifiers,
C	     which modify the print command according to DCL rules.)
C
C	V. PARSE INPUT STRING FOR FILE SPECS, AND LOCAL QUALIFIERS, 
C	   AND APPEND VALID PARAMETERS AND QUALIFIERS ONTO THE PRINT
C	   COMMAND STRING.
C	    Parse the input for each parameter entry (filespecs, with
C	    possible wildcards), and set local flags for the presence
C 	    of local qualifiers. Build local prefix strings, according
C	    to local formatting qualifiers present, that apply only to
C	    current input parameter. Any errors here will cause skipping
C	    of the invalid parameter item.
C
C	    A. Unwind the input filespec parameter (in case it 
C	       contains wildcards), determining all the specific files 
C	       it references. Make sure the files are present, and are
C	       text files (by verifying that the file type isn't known
C	       to be unprintable.)  If FORMATTING is selected, and any
C	       good files are found in the parameter, then let the
C	       (file) parameter be included in the PRINT command string.
C	       If NOFORMATTING is selected, then skip the parameter, and
C              do not include it in the PRINT command string. 
C
C	    B. Determine if FORMATTING has been selected for this file,
C	       and do the following:
C
C	       If formatting is selected, then:
C
C		1. Attempt to open the input file. If problems, go 
C		   back and unwind the next file from the current
C		   filespec parameter.
C		2. Attempt to open the output file with the same name,
C		   type, and version number as the input file. If it
C		   already exists, or can't be opened, go back and
C		   unwind the next file from the current filespec
C		   parameter.
C		3. Determine what the current active margin level is
C		   for this file (a parameter item).
C		4. If pitch is not explicitly specified, perform auto-
C		   sizing (i.e. pick a pitch that will allow full page
C		   printing for this specific file, and build the file
C		   specific prefix string with this pitch, current
C		   active margin, and draft-mode if needed.)  Auto-
C		   sizing may use a margin different from one speci-
C		   fied by a /MARGIN qualifier for this file. It may
C		   also shut off /LETTER, if required pitch precludes
C		   use of generic font-pitch of active font. 
C		5. Build tab prefix string from current active margin.
C		6. Copy each file over to the system print directory,
C		   SYS$SYSPRINT:, with global-, local-, file-, and
C	           tab-prefix strings inserted into the first record,
C		   and global-suffix strings appended as the last 
C		   record of the file.
C		7. Close the input and output files
C
C	    Else, if UNFORMATTED is selected, then:
C
C		1. Append on valid parameters, and associated qualifiers
C	           to the PRINT command line.
C	       
C       VI. USE LIB$DO_COMMAND TO INVOKE USERPRINT.COM, WHICH WILL
C	    INVOKE THE PRINT COMMAND LINE GENERATED BY THIS PROGRAM.
C	     If the user's input print command specified any good files,
C	     then invoke the USERPRINT procedure to queue the print job,
C            using the PRINT command line generated by this program, and
C	     exit.  Else, tell the user that no jobs were queued, and
C	     exit.
C
C
C---------INITIALIZE VARIABLES, ARRAYS, AND PARAMETERS.
C
	EXTERNAL   SYS$GETJPI,CLI$PRESENT,CLI$GET_VALUE,
     +             CLI$_PRESENT,CLI$_ABSENT,CLI$_NEGATED,
     +             CLI$_CONCAT,CLI$_LOCPRES,CLI$_LOCNEG,
     +             BUILD_COM,CHAR2INT,LIB$FIND_FILE,LIB$DO_COMMAND,
     +             BUILD_FORMAT,BUILD_TAB,LIB$DISABLE_CTRL,
     +             SYS$ASSIGN,ENABLE_CTRLY,CTRLY_HANDLER
C
	INTEGER*4  SYS$GETJPI,CLI$PRESENT,CLI$GET_VALUE,STATUS,
     +             TERM_BUF_ADD,RET_LEN_ADD,RET_LEN,IOSTAT,
     +             CONTEXT,TERM_CHAN,SYS$ASSIGN,LIB$DISABLE_CTRL
	LOGICAL*4  ERROR_FLAG
	INTEGER*2  ITMLST(8),ITEM_CODE,TERM_BUF_LEN,JPI_STATUS,IOSB(4),
     +             QUEUE_LEN,REC_LEN,START_PT,END_PT,PARAM_ITEM_LEN,
     +             SHORT_FS_LEN,FULL_FS_LEN,GPREFIX_END_PT,
     +             LPREFIX_END_PT,FPREFIX_END_PT,SUFFIX_END_PT,
     +             TPREFIX_END_PT,MARGIN_STR_LEN,ACT_MARGIN_STR_LEN,
     +             DEF_MARGIN_STR_LEN,ACT_MARGIN,DEF_MARGIN,
     +             TSUFFIX_END_PT,VALUE,SHORT_PARAM_LEN,I,NUMBER
	CHARACTER  USERS_TERMINAL*7,AL_DEVICES*150,JF_DEVICES*60,
     +             HF_DEVICES*30,CAD_DEVICES*6,QUEUE*30,
     +             FULL_FILESPEC*80,COM_LINE*260,SHORT_PARAM*80,
     +             AFTER_STR*20,COPIES_STR*20,FONT_STR*20,
     +             JOB_COUNT_STR*20,MARGIN_STR*20,NAME_STR*20,
     +             PAGE_COUNT_STR*20,PITCH_STR*20,PRIORITY_STR*20,
     +             RESULT_FS*80,DEFAULT_FS*80,RELATED_FS*80,
     +             PARAM_ITEM*80,SHORT_FILESPEC*30,
     +             ACTIVE_MARGIN_STR*5,DEFAULT_MARGIN_STR*5,
     +             MARGIN_PREFIX*10,FILE_TYPE*3
	BYTE       FILE_BUFFER(250),GPREFIX_STR(50),LPREFIX_STR(50),
     +             FPREFIX_STR(20),SUFFIX_STR(50),TPREFIX_STR(55),
     +             TSUFFIX_STR(55)
	LOGICAL*1  AL_FLAG,JF_FLAG,HF_FLAG,CAD_FLAG,LA100_FLAG,
     +             LA120_FLAG,VERSATEC_FLAG,FILE_EXISTS,MORE_FILES,
     +             AFTER_FLAG,COPIES_FLAG,DRAFT_FLAG,FLAG_PAGE_FLAG,
     +             NOFLAG_PAGE_FLAG,FONT_FLAG,FORM_FLAG,HEADER_FLAG,
     +             NOHEADER_FLAG,JOB_COUNT_FLAG,LETTER_FLAG,MARGIN_FLAG,
     +             NAME_FLAG,PITCH_FLAG,PRIORITY_FLAG,SPACE_FLAG,
     +             COPIES_LFLAG,FLAG_PAGE_LFLAG,NOFLAG_PAGE_LFLAG,
     +             FONT_LFLAG,HEADER_LFLAG,NOHEADER_LFLAG,MARGIN_LFLAG,
     +             PAGE_COUNT_LFLAG,PITCH_LFLAG,SPACE_LFLAG,GOOD_PARAMS,
     +             DEV_ENTRY_FLAG,DRAFT_LFLAG,UNWINDING,SKIP_PARAM,
     +             UNRECDEV_FLAG,CTRLY_EXIT_FLAG,NOTIFY_FLAG,
     +             DELETE_FLAG,DELETE_LFLAG,FONT_10,FONT_12,
     +             COM_LINE_ENTRIES,FONT_10_FLAG,FONT_12_FLAG
C
	COMMON     /HANDLER/TERM_CHAN,FORM_FLAG,CTRLY_EXIT_FLAG
C
	INCLUDE    '($FORIOSDEF)'
	INCLUDE    '($LIBCLIDEF)'
C
	PARAMETER  JPI$_TERMINAL = 797
	PARAMETER  SS$_NORMAL = 1
	PARAMETER  ESCAPE          = CHAR(27)
	PARAMETER  C_R             = CHAR(13)
C
C---------Set up terminal escape sequences that the LA100 and LA120 
C	understand. (Font locations, valid only for LA100's, are 
C	assumed.)
C
	PARAMETER  COURIER10       = ESCAPE//'[10m'
	PARAMETER  UPPER           = ESCAPE//'[11m'
	PARAMETER  ORATOR10        = ESCAPE//'[12m'
	PARAMETER  LOWER           = ESCAPE//'[13m'
	PARAMETER  GOTHIC12        = ESCAPE//'[14m'
	PARAMETER  DRAFT_MODE      = ESCAPE//'[0"z'
	PARAMETER  LETTER_MODE     = ESCAPE//'[2"z'
	PARAMETER  HP5             = ESCAPE//'[5w'
	PARAMETER  HP6             = ESCAPE//'[6w'
	PARAMETER  HP6_6           = ESCAPE//'[7w'
	PARAMETER  HP8_25          = ESCAPE//'[8w'
	PARAMETER  HP10            = ESCAPE//'[1w'
	PARAMETER  HP12            = ESCAPE//'[2w'
	PARAMETER  HP13_2          = ESCAPE//'[3w'
	PARAMETER  HP16_5          = ESCAPE//'[4w'
	PARAMETER  VP4LPI          = ESCAPE//'[6z'
	PARAMETER  VP6LPI          = ESCAPE//'[0z'
	PARAMETER  VM1_40          = ESCAPE//'[1;40r'
	PARAMETER  VM1_64          = ESCAPE//'[1;64r'
	PARAMETER  CLEAR_TAB       = ESCAPE//'[2g'
	PARAMETER  USCHAR          = ESCAPE//'(B'
C
C---------Set up escape sequences that return LA100 and LA120 terminals 
C	back to an acceptable default condition.
C
	PARAMETER  FONT_SUFFIX     = COURIER10
	PARAMETER  VPITCH_SUFFIX   = VP6LPI
	PARAMETER  QUALITY_SUFFIX  = DRAFT_MODE
	PARAMETER  VMARGIN_SUFFIX  = VM1_64
	PARAMETER  LA100_HP_SUFFIX = HP16_5
	PARAMETER  LA120_HP_SUFFIX = HP16_5
	PARAMETER  LA100_HM_SUFFIX = ESCAPE//'[8;210s' ! 8 Left/210 Right
	PARAMETER  LA120_HM_SUFFIX = ESCAPE//'[1;210s' ! 1 Left/210 Right
	PARAMETER  CLU_SUFFIX      = ESCAPE//'[0m'     ! Clear Underline
C
	PARAMETER  LA100_TAB_SUFFIX=
     +   ESCAPE//'[16;24;32;40;48;56;64;72;80;88;96;104;112;120;128u'
C
	PARAMETER  LA120_TAB_SUFFIX= 
     +   ESCAPE//'[9;17;25;33;41;49;57;65;73;81;89;97;105;113;121;129u'
C
	PARAMETER  LA100_SUFFIX    = FONT_SUFFIX//QUALITY_SUFFIX//
     +LA100_HP_SUFFIX//LA100_HM_SUFFIX//VMARGIN_SUFFIX//VPITCH_SUFFIX//
     +CLU_SUFFIX//CLEAR_TAB//C_R
C
	PARAMETER  LA120_SUFFIX    = LA120_HP_SUFFIX//LA120_HM_SUFFIX//
     +VMARGIN_SUFFIX//VPITCH_SUFFIX//USCHAR//CLEAR_TAB//C_R
C
C--------------------------------------------------------------------
C
	DATA       TERM_BUF_LEN/7/, MORE_FILES/.TRUE./, 
     +             DEFAULT_FS(1:4)/'.LIS'/,
     +             MARGIN_PREFIX(1:1)/ESCAPE/, MARGIN_PREFIX(2:2)/'['/
C
C---------Set up a master list describing which terminals are in which
C	buildings. "AL" is Aloha 3; "JF" is Jones Farm 1; "HF" is
C	Hawthorn Farms; "CAD" are any CAD users, who as a class don't
C	have printing rights on our system, and are lumped together.
C
	DATA       AL_DEVICES(1:30)  /'TTA0: TTA1: TTA2: TTA3: TTA4: '/,
     +             AL_DEVICES(31:60) /'TTA5: TTA6: TTA7: TTB0: TTB1: '/,
     +             AL_DEVICES(61:90) /'TTB2: TTB3: TTB4: TTB5: TTB6: '/,
     +             AL_DEVICES(91:120)/'TTB7: TTC0: TTC1: TTC2: TTC7: '/,
     +             AL_DEVICES(121:150)/'TTD0: TTD1: ***** ***** ***** '/
	DATA       JF_DEVICES(1:30)  /'TTC3: TTC4: TTC6: TTD2: TTD3: '/,
     +             JF_DEVICES(31:60) /'TTD4: TTD7: ***** ***** ***** '/
	DATA       HF_DEVICES(1:30)  /'TTD5: TTD6: ***** ***** ***** '/
	DATA       CAD_DEVICES(1:6)  /'TTC5: '/
C
C
C---------First, look to see what building the user is in.
C	  (Unrecognized devices go to AL-3 by default.)
C
	EQUIVALENCE (ITMLST(1),TERM_BUF_LEN), (ITMLST(2),ITEM_CODE),
     +              (ITMLST(3),TERM_BUF_ADD), (ITMLST(5),RET_LEN_ADD),
     +              (IOSB(1),JPI_STATUS)
C
	ITEM_CODE  = JPI$_TERMINAL
	TERM_BUF_ADD = %LOC(USERS_TERMINAL)
	RET_LEN_ADD = %LOC(RET_LEN)
C
	STATUS = SYS$GETJPI (,,,ITMLST,IOSB,,)
C
	IF (STATUS .NE. SS$_NORMAL .OR. JPI_STATUS .NE. SS$_NORMAL) THEN
	   TYPE *,
     +      '%USERPRINT-F-GETJPI, Failure to get login-device number'
	   CALL LIB$STOP (%VAL(JPI_STATUS))
	END IF
C	   
	IF (INDEX(AL_DEVICES,USERS_TERMINAL(1:5)) .NE. 0) THEN
	   AL_FLAG = .TRUE.
	ELSE IF (INDEX(JF_DEVICES,USERS_TERMINAL(1:5)) .NE. 0) THEN
	   JF_FLAG = .TRUE.
	ELSE IF (INDEX(HF_DEVICES,USERS_TERMINAL(1:5)) .NE. 0) THEN
	   HF_FLAG = .TRUE.
	ELSE IF (INDEX(CAD_DEVICES,USERS_TERMINAL(1:5)) .NE. 0) THEN
	   CAD_FLAG = .TRUE.
	   TYPE *,  '%USERPRINT-I-NOCADPRNT,',
     +      ' MPD CAD has no printing device on this system'
	   CALL EXIT
	ELSE
	   UNRECDEV_FLAG = .TRUE.
	END IF
C
C---------Next, determine what type of printer device the user needs.
C	  Handle the case where a queue is specified, as well as a 
C         specific device requested or implied by location. (Batch job
C	  PRINT commands go to SYS$TERMINAL by default.)
C
	IF (CLI$PRESENT('DEVICE') .EQ. %LOC(CLI$_PRESENT)) THEN
	   CALL CLI$GET_VALUE('DEVICE',QUEUE)
	   DEV_ENTRY_FLAG = .TRUE.
	ELSE IF (CLI$PRESENT('QUEUE') .EQ. %LOC(CLI$_PRESENT)) THEN
	   CALL CLI$GET_VALUE('QUEUE',QUEUE)
	   DEV_ENTRY_FLAG = .TRUE.
	END IF
C
	IF (DEV_ENTRY_FLAG) THEN
	   IF      ((INDEX(QUEUE,'AL3LA100')     .NE. 0) .OR.
     +              (INDEX(QUEUE,'TTD1')         .NE. 0)) THEN
	      QUEUE(1:4) = 'TTD1'
	      QUEUE_LEN  = 4
	      LA100_FLAG = .TRUE.
C
	   ELSE IF ((INDEX(QUEUE,'JF1LA100')     .NE. 0) .OR.
     +              (INDEX(QUEUE,'TTD3')         .NE. 0)) THEN
	      QUEUE(1:4) = 'TTD3'
	      QUEUE_LEN  = 4
	      LA100_FLAG = .TRUE.
C
	   ELSE IF ((INDEX(QUEUE,'LA120')        .NE. 0) .OR.
     +              (INDEX(QUEUE,'SYS$TERMINAL') .NE. 0)) THEN
	      QUEUE(1:12) = 'SYS$TERMINAL'
	      QUEUE_LEN  = 12
	      LA120_FLAG = .TRUE.
C
	   ELSE IF ((INDEX(QUEUE,'VERSATEC')     .NE. 0) .OR.
     +              (INDEX(QUEUE,'SYS$PRINT')    .NE. 0)) THEN
	      QUEUE(1:9) = 'SYS$PRINT'
	      QUEUE_LEN  = 9
	      VERSATEC_FLAG = .TRUE.
C
	   ELSE
	      TYPE *, '%USERPRINT-E-NOSUCHDEV, No such device or queue'
	      CALL EXIT
	   END IF
C
	ELSE IF (AL_FLAG) THEN
	   LA100_FLAG = .TRUE.
	   QUEUE(1:4) = 'TTD1'
	   QUEUE_LEN  = 4
C
	ELSE IF (JF_FLAG) THEN
	   LA100_FLAG = .TRUE.
	   QUEUE(1:4) = 'TTD3'
	   QUEUE_LEN  = 4
C
	ELSE IF (HF_FLAG) THEN
	   LA120_FLAG = .TRUE.
	   QUEUE(1:12) = 'SYS$TERMINAL'
	   QUEUE_LEN  = 12
C
	ELSE  !UNRECDEV_FLAG
	   TYPE *, '%USERPRINT-I-UNRECDEV, No queue specified,',
     +      ' defaulting to SYS$TERMINAL in Aloha 3.'
	   QUEUE(1:12) = 'SYS$TERMINAL'
	   QUEUE_LEN  = 12
	   LA120_FLAG = .TRUE.
	END IF
C
C---------Set up the interupt handler here, if the user has a terminal
C         assigned to his process.
C
	IF (.NOT. UNRECDEV_FLAG) THEN
C
	   STATUS = SYS$ASSIGN (USERS_TERMINAL(1:5),TERM_CHAN,,)
	   IF (STATUS .NE. 1) THEN
	      TYPE *, '%USERPRINT-F-ASNCHFAIL, Cannot assign channel'
	      CALL LIB$STOP (%VAL(STATUS))
	   END IF
C
	   CALL ENABLE_CTRLY
C
	   STATUS = LIB$DISABLE_CTRL (LIB$M_CLI_CTRLY,)
	   IF (STATUS .NE. 1) THEN
	      TYPE *,
     +         '%USERPRINT-F-DSBLCTRLY, Cannot disable <CTRL-Y>'
	      CALL LIB$STOP (%VAL(STATUS))
	   END IF
C
	END IF
C
C---------Inspect the string for global symbols, and get values and/or
C         set flags for those specified. Make special note of a negated
C         FORMATTING flag, and interpret things accordingly. Selecting
C         the VERSATEC will automatically disable formatting.
C
	IF (CLI$PRESENT('FORMATTING') .EQ. %LOC(CLI$_NEGATED) .OR.
     +	 VERSATEC_FLAG)  THEN
	   FORM_FLAG = .FALSE.
	ELSE
	   FORM_FLAG = .TRUE.
	END IF
C
	IF (CLI$PRESENT('AFTER')     .EQ. %LOC(CLI$_PRESENT))  THEN
	   IF (.NOT. LA120_FLAG) THEN
	      CALL CLI$GET_VALUE ('AFTER',AFTER_STR)
	      AFTER_FLAG = .TRUE.
	   ELSE
	      TYPE *, '%USERPRINT-E-NOAFTERSUP, SYS$TERMINAL queue',
     +         ' doesn''t support /AFTER qualifier'
	   CALL EXIT
	   END IF
	END IF
C
	IF (CLI$PRESENT('COPIES')    .EQ. %LOC(CLI$_PRESENT)) THEN
	   CALL CLI$GET_VALUE ('COPIES',COPIES_STR)
	   CALL CHAR2INT (COPIES_STR,VALUE,ERROR_FLAG)
	   IF (ERROR_FLAG .OR. VALUE .LT. 1 .OR. VALUE .GT. 10) THEN
	      TYPE *, '%USERPRINT-E-COPYVAL,',
     +         ' Improper COPIES syntax, or value out of range'
	      CALL EXIT
	   END IF
	   COPIES_FLAG = .TRUE.
	END IF
C
	IF (CLI$PRESENT('DELETE') .EQ. %LOC(CLI$_PRESENT)) THEN
	   IF (.NOT. FORM_FLAG) THEN
	      DELETE_FLAG = .TRUE.
	   ELSE
	      TYPE *, '%USERPRINT-E-INVDELE,',
     +         ' Cannot use /DELETE with /FORMATTING'
	      CALL EXIT
	   END IF
	END IF
C
	IF (CLI$PRESENT('DRAFT') .EQ. %LOC(CLI$_PRESENT) .AND.
     +   FORM_FLAG .AND. LA100_FLAG)  DRAFT_FLAG = .TRUE.
C
	IF (CLI$PRESENT('FLAG_PAGE') .EQ. %LOC(CLI$_PRESENT))  THEN
	   FLAG_PAGE_FLAG = .TRUE.
	ELSE IF (CLI$PRESENT('FLAG_PAGE') .EQ. %LOC(CLI$_NEGATED)) THEN
	   NOFLAG_PAGE_FLAG = .TRUE.
	END IF
C
	IF (CLI$PRESENT('FONT')      .EQ. %LOC(CLI$_PRESENT) .AND.
     +     FORM_FLAG)  THEN
	   IF (LA100_FLAG) THEN
	      CALL CLI$GET_VALUE ('FONT',FONT_STR)
	      FONT_FLAG = .TRUE.
	      IF (FONT_STR(1:9) .EQ. 'COURIER10') THEN
	         FONT_10_FLAG = .TRUE.
	      ELSE IF (FONT_STR(1:5) .EQ. 'UPPER') THEN
	         FONT_10_FLAG = .TRUE.
	         CALL BUILD_FORMAT
     +            (GPREFIX_STR,UPPER,GPREFIX_END_PT)
	      ELSE IF (FONT_STR(1:8) .EQ. 'ORATOR10') THEN
	         FONT_10_FLAG = .TRUE.
                 CALL BUILD_FORMAT
     +            (GPREFIX_STR,ORATOR10//VP4LPI//VM1_40,GPREFIX_END_PT) 
	      ELSE IF (FONT_STR(1:5) .EQ. 'LOWER') THEN
	         FONT_10_FLAG = .TRUE.
	         CALL BUILD_FORMAT
     +            (GPREFIX_STR,LOWER,GPREFIX_END_PT) 
	      ELSE IF (FONT_STR(1:8) .EQ. 'GOTHIC12') THEN
	         FONT_12_FLAG = .TRUE.
	         CALL BUILD_FORMAT
     +            (GPREFIX_STR,GOTHIC12,GPREFIX_END_PT) 
	      ELSE
	         TYPE *, '%USERPRINT-E-INVFONT, invalid or unsupported',
     +            ' font type /',FONT_STR(1:10),'/'
	         CALL EXIT
	      END IF
	   ELSE
	      TYPE *, '%USERPRINT-I-NOFONTSUP,',
     +        ' font selection not supported for this device'
	   END IF
	END IF
C
	IF (CLI$PRESENT('HEADER')    .EQ. %LOC(CLI$_PRESENT))  THEN
	   HEADER_FLAG = .TRUE.
	ELSE IF (CLI$PRESENT('HEADER') .EQ. %LOC(CLI$_NEGATED)) THEN
	   NOHEADER_FLAG = .TRUE.
	END IF
C
	IF (CLI$PRESENT('JOB_COUNT') .EQ. %LOC(CLI$_PRESENT)) THEN
	   CALL CLI$GET_VALUE ('JOB_COUNT',JOB_COUNT_STR)
	   CALL CHAR2INT (JOB_COUNT_STR,VALUE,ERROR_FLAG)
	   IF (ERROR_FLAG .OR. VALUE .LT. 1 .OR. VALUE .GT. 10) THEN
	      TYPE *, '%USERPRINT-E-JOBCNTVAL,',
     +         ' Improper JOB_COUNT syntax, or value out of range'
	      CALL EXIT
	   END IF
	   JOB_COUNT_FLAG = .TRUE.
	END IF
C
	IF (CLI$PRESENT('LETTER')    .EQ. %LOC(CLI$_PRESENT) .AND.
     +     FORM_FLAG) THEN
	   IF (DRAFT_FLAG .AND. LA100_FLAG) THEN
	      TYPE *,'%USERPRINT-E-CONFQUAL, Conflicting qualifiers,',
     +         ' /DRAFT/LETTER/'
	      CALL EXIT
	   ELSE IF (LA100_FLAG) THEN
              CALL BUILD_FORMAT (GPREFIX_STR,LETTER_MODE,GPREFIX_END_PT) 
	      LETTER_FLAG = .TRUE.
	   ELSE
	      TYPE *, '%USERPRINT-I-NOLETSUP,',
     +         ' device does not support LETTER quality printing'
	   END IF
	END IF
C
	IF (CLI$PRESENT('NAME')      .EQ. %LOC(CLI$_PRESENT)) THEN
	   CALL CLI$GET_VALUE ('NAME',NAME_STR)
	   NAME_FLAG = .TRUE.
	END IF
C
	IF (CLI$PRESENT('NOTIFY') .EQ. %LOC(CLI$_PRESENT))
     +   NOTIFY_FLAG = .TRUE.
C
	IF (CLI$PRESENT('PITCH')     .EQ. %LOC(CLI$_PRESENT) .AND.
     +   FORM_FLAG) THEN
	   CALL CLI$GET_VALUE ('PITCH',PITCH_STR)
	   PITCH_FLAG = .TRUE.
	   IF (PITCH_STR .EQ. '5') THEN
	      CALL BUILD_FORMAT (GPREFIX_STR,HP5,GPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 1
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '5'
	   ELSE IF (PITCH_STR .EQ. '6') THEN
	      CALL BUILD_FORMAT (GPREFIX_STR,HP6,GPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 1
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '6'
	   ELSE IF (PITCH_STR .EQ. '6.6') THEN
	      CALL BUILD_FORMAT (GPREFIX_STR,HP6_6,GPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 1
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '6'
	   ELSE IF (PITCH_STR .EQ. '8.25') THEN
	      CALL BUILD_FORMAT (GPREFIX_STR,HP8_25,GPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 1
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '8'
	   ELSE IF (PITCH_STR .EQ. '10') THEN
	      CALL BUILD_FORMAT (GPREFIX_STR,HP10,GPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 2
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '10'
	   ELSE IF (PITCH_STR .EQ. '12') THEN
	      CALL BUILD_FORMAT (GPREFIX_STR,HP12,GPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 2
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '12'
	   ELSE IF (PITCH_STR .EQ. '13.2') THEN
	      CALL BUILD_FORMAT (GPREFIX_STR,HP13_2,GPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 2
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '13'
	   ELSE IF (PITCH_STR .EQ. '16.5') THEN
	      CALL BUILD_FORMAT (GPREFIX_STR,HP16_5,GPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 1
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '8'
	   ELSE
	      TYPE *, '%USERPRINT-E-PITCHVAL,',
     +         ' Improper PITCH syntax, or value out of range'
	      CALL EXIT
	   END IF
	ELSE  !Set the default value for LA100's (LA120's will overide)
	   DEF_MARGIN_STR_LEN = 2
	   DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '10'
	END IF
C
	IF (CLI$PRESENT('MARGIN')    .EQ. %LOC(CLI$_PRESENT) .AND.
     +     FORM_FLAG) THEN
	   MARGIN_FLAG = .TRUE.
	   CALL CLI$GET_VALUE ('MARGIN',MARGIN_STR)
	   CALL CHAR2INT (MARGIN_STR,VALUE,ERROR_FLAG)
	   IF (VALUE .GT. 80 .OR. ERROR_FLAG) THEN
	      TYPE *, '%USERPRINT-E-MARGINVAL,',
     +         ' Improper MARGIN syntax, or value out of range'
	      CALL EXIT
	   ELSE IF (VALUE .LE. 0) THEN
	      MARGIN_STR(1:2) = '1 '
	   END IF
	   MARGIN_STR_LEN = INDEX(MARGIN_STR,' ') - 1
	   MARGIN_PREFIX(3:3+MARGIN_STR_LEN) =
     +      MARGIN_STR(1:MARGIN_STR_LEN)//'s'
	   CALL BUILD_FORMAT (GPREFIX_STR,
     +      MARGIN_PREFIX(1:3+MARGIN_STR_LEN),GPREFIX_END_PT)
	END IF
C
	IF (CLI$PRESENT('PRIORITY')  .EQ. %LOC(CLI$_PRESENT)) THEN
	   CALL CLI$GET_VALUE ('PRIORITY',PRIORITY_STR)
	   CALL CHAR2INT (PRIORITY_STR,VALUE,ERROR_FLAG)
	   IF (ERROR_FLAG .OR. VALUE .LT. 0 .OR. VALUE .GT. 31) THEN
	      TYPE *, '%USERPRINT-E-PRIOVAL,',
     +         ' Improper PRIORITY syntax, or value out of range'
	      CALL EXIT
	   END IF
	   PRIORITY_FLAG = .TRUE.
	END IF
C
	IF (CLI$PRESENT('SPACE')     .EQ. %LOC(CLI$_PRESENT))
     +    SPACE_FLAG = .TRUE.
C
C---------If formatted output, build suffix, and other strings,
C         appropriate for device selected.
C
	IF (LA100_FLAG .AND. FORM_FLAG) THEN
	   CALL BUILD_FORMAT (SUFFIX_STR,LA100_SUFFIX,SUFFIX_END_PT)
	   CALL BUILD_FORMAT 
     +      (TSUFFIX_STR,LA100_TAB_SUFFIX,TSUFFIX_END_PT)
C
	ELSE IF (LA120_FLAG .AND. FORM_FLAG) THEN
	   CALL BUILD_FORMAT (SUFFIX_STR,LA120_SUFFIX,SUFFIX_END_PT)
	   CALL BUILD_FORMAT 
     +      (TSUFFIX_STR,LA120_TAB_SUFFIX,TSUFFIX_END_PT)
	   DEF_MARGIN_STR_LEN = 1
	   DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '1'
	END IF
C
C---------Start building the command string, using syntax appropriate 
C         for FORMATTING flag value.
C
	IF (FORM_FLAG) THEN
	   COM_LINE(1:15) = 'PRINT/DELE/QUE='
	   END_PT = 15 + QUEUE_LEN
	   COM_LINE(16:END_PT) = QUEUE(1:QUEUE_LEN)
	   START_PT = END_PT + 1
	ELSE
	   COM_LINE(1:10) = 'PRINT/QUE='
	   END_PT = 10 + QUEUE_LEN
	   COM_LINE(11:END_PT) = QUEUE(1:QUEUE_LEN)
	   START_PT = END_PT + 1
	END IF
C
C---------Begin building the command string with global qualifiers.
C         (Ignore non-DCL qualifiers specific to formatting.)
C
	IF (AFTER_FLAG)
     +   CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/AFT=',AFTER_STR)
C
	IF (COPIES_FLAG)
     +     CALL BUILD_COM
     +      (COM_LINE,START_PT,END_PT,'/COP=',COPIES_STR)
C
	IF (DELETE_FLAG)
     +     CALL BUILD_COM
     +      (COM_LINE,START_PT,END_PT,'/','DELE ')
C
	IF (FLAG_PAGE_FLAG) THEN
           CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','FLAG ')
	ELSE IF (NOFLAG_PAGE_FLAG) THEN
           CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','NOFLAG ')
	END IF
C
	IF (HEADER_FLAG) THEN
           CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','HEAD ')
	ELSE IF (NOHEADER_FLAG) THEN
           CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','NOHEAD ')
	END IF
C
	IF (JOB_COUNT_FLAG)
     +     CALL BUILD_COM
     +      (COM_LINE,START_PT,END_PT,'/JOB=',JOB_COUNT_STR)
C
	IF (NAME_FLAG)
     +     CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/NAM=',NAME_STR)
C
	IF (NOTIFY_FLAG)
     +     CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','NOTIF ')
C
	IF (PRIORITY_FLAG) THEN
           CALL BUILD_COM
     +      (COM_LINE,START_PT,END_PT,'/PRIO=',PRIORITY_STR)
	ELSE
           CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/PRIO=','4 ')
	END IF
C
	IF (SPACE_FLAG)
     +   CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','SPACE ')
C
C---------If formatting is desired, build the front-end of the parameter
C         list specifying the system disk print-spooling directory,
C         SYS$SYSPRINT. If no formatting, then put a space between the
C         command qualifiers, and the first parameter item.
C
        IF (FORM_FLAG)  THEN
	   CALL BUILD_COM
     +      (COM_LINE,START_PT,END_PT,' ','SYS$SYSPRINT: ')
	ELSE
	   COM_LINE(START_PT:START_PT) = ' '
	   START_PT = START_PT + 1
	   END_PT = END_PT + 1
	END IF
C
C---------If a Control-Y Exit-Flag, exit now.
C
	IF (CTRLY_EXIT_FLAG) CALL EXIT
C
C---------Start retrieving the files from the parameter list, keeping
C	  track of local qualifier flags and values attached to each
C         qualifier. (Ignore qualifiers previously attached to the
C         command name identified above.)
C
	DO WHILE (MORE_FILES)
	   STATUS = CLI$PRESENT ('FILESPEC')
	   STATUS = CLI$GET_VALUE('FILESPEC',PARAM_ITEM)
	   IF (STATUS .EQ. SS$_NORMAL .OR.
     +         STATUS .EQ. %LOC(CLI$_CONCAT)) THEN
	      CALL CLEAR_FORMAT (LPREFIX_STR,LPREFIX_END_PT,50)
	      MORE_FILES = .TRUE.
	      SKIP_PARAM = .FALSE.
	   ELSE
	      MORE_FILES = .FALSE.
	      GO TO 600
	   END IF
C
C---------Extract the short form of the current parameter.
C
	   I = MAX
     +      ((INDEX(PARAM_ITEM,':')),(INDEX(PARAM_ITEM,']')))
	   SHORT_PARAM = PARAM_ITEM(I+1:)
	   SHORT_PARAM_LEN = INDEX(SHORT_PARAM,' ') - 1
	   PARAM_ITEM_LEN = INDEX(PARAM_ITEM,' ') - 1
C
C---------Extract local qualifiers and values, and check for proper
C         values.
C
	IF (CLI$PRESENT('COPIES')    .EQ. %LOC(CLI$_LOCPRES) .AND.
     +   .NOT. COPIES_FLAG) THEN
	   CALL CLI$GET_VALUE ('COPIES',COPIES_STR)
	   CALL CHAR2INT (COPIES_STR,VALUE,ERROR_FLAG)
	   IF (ERROR_FLAG .OR. VALUE .LT. 1 .OR. VALUE .GT. 10) THEN
	      TYPE *, '%USERPRINT-E-COPYVAL,',
     +         ' Improper COPIES syntax, or value out of range'
	      COPIES_LFLAG = .FALSE.
	      SKIP_PARAM = .TRUE.
	      GO TO 500
	   END IF
	   COPIES_LFLAG = .TRUE.
	ELSE
	   COPIES_LFLAG = .FALSE. 
	END IF
C
	IF (CLI$PRESENT('DELETE') .EQ. %LOC(CLI$_LOCPRES) .AND. .NOT.
     +   DELETE_FLAG)  THEN
	   IF (.NOT. FORM_FLAG) THEN
	      DELETE_LFLAG = .TRUE.
	   ELSE
	      TYPE *, '%USERPRINT-E-INVDELE,',
     +         ' Cannot use /DELETE with /FORMATTING'
	      DELETE_LFLAG = .FALSE.
	      SKIP_PARAM = .TRUE.
	      GO TO 500
	   END IF
	ELSE
	   DELETE_LFLAG = .FALSE.
	END IF	   
C
	IF (CLI$PRESENT('DRAFT') .EQ. %LOC(CLI$_LOCPRES) .AND. .NOT.
     +   DRAFT_FLAG .AND. .NOT. LETTER_FLAG .AND. FORM_FLAG) THEN
           DRAFT_LFLAG = .TRUE.
	ELSE
           DRAFT_LFLAG = .FALSE.
	END IF	   
C
	IF (CLI$PRESENT('FLAG_PAGE') .EQ. %LOC(CLI$_LOCPRES) .AND. .NOT.
     +   FLAG_PAGE_FLAG .AND. .NOT. NOFLAG_PAGE_FLAG) THEN
	   FLAG_PAGE_LFLAG = .TRUE.
	   NOFLAG_PAGE_LFLAG = .FALSE.
	ELSE IF (CLI$PRESENT('FLAG_PAGE') .EQ. %LOC(CLI$_LOCNEG) .AND.
     +   .NOT. FLAG_PAGE_FLAG .AND. .NOT. NOFLAG_PAGE_FLAG) THEN
	   NOFLAG_PAGE_LFLAG = .TRUE.
	   FLAG_PAGE_LFLAG = .FALSE.
	ELSE
	   FLAG_PAGE_LFLAG = .FALSE.
	   NOFLAG_PAGE_LFLAG = .FALSE.
	END IF
C
	IF (CLI$PRESENT('FONT') .EQ. %LOC(CLI$_LOCPRES) .AND. .NOT.
     +   FONT_FLAG .AND. FORM_FLAG) THEN
	   IF (LA100_FLAG) THEN
	      CALL CLI$GET_VALUE ('FONT',FONT_STR)
	      FONT_10_FLAG = .FALSE.
	      FONT_12_FLAG = .FALSE.
	      FONT_LFLAG = .TRUE.
	      IF (FONT_STR(1:9) .EQ. 'COURIER10') THEN
	         FONT_10_FLAG = .TRUE.
	      ELSE IF (FONT_STR(1:5) .EQ. 'UPPER') THEN
	         FONT_10_FLAG = .TRUE.
	         CALL BUILD_FORMAT (LPREFIX_STR,UPPER,LPREFIX_END_PT) 
	      ELSE IF (FONT_STR(1:8) .EQ. 'ORATOR10') THEN
	         FONT_10_FLAG = .TRUE.
                 CALL BUILD_FORMAT
     +            (LPREFIX_STR,ORATOR10//VP4LPI//VM1_40,LPREFIX_END_PT) 
	      ELSE IF (FONT_STR(1:5) .EQ. 'LOWER') THEN
	         FONT_10_FLAG = .TRUE.
	         CALL BUILD_FORMAT (LPREFIX_STR,LOWER,LPREFIX_END_PT) 
	      ELSE IF (FONT_STR(1:8) .EQ. 'GOTHIC12') THEN
	         FONT_12_FLAG = .TRUE.
	         CALL BUILD_FORMAT
     +           (LPREFIX_STR,GOTHIC12,LPREFIX_END_PT) 
	      ELSE
	         TYPE *, '%USERPRINT-E-INVFONT, invalid or unsupported',
     +            ' font type /',FONT_STR(1:10),'/'
	         SKIP_PARAM = .TRUE.
	         FONT_LFLAG = .FALSE.
	         GO TO 500
	      END IF
	   ELSE
	      TYPE *, '%USERPRINT-I-NOFONTSUP,',
     +        ' font selection not supported for this device'
	      FONT_LFLAG = .FALSE.
	   END IF
	ELSE
	   FONT_LFLAG = .FALSE.
	END IF
C
	IF (CLI$PRESENT('HEADER') .EQ. %LOC(CLI$_LOCPRES) .AND. .NOT.
     +   HEADER_FLAG .AND. .NOT. NOHEADER_FLAG) THEN
	   HEADER_LFLAG = .TRUE.
	   NOHEADER_LFLAG = .FALSE.
	ELSE IF (CLI$PRESENT('HEADER') .EQ. %LOC(CLI$_LOCNEG) .AND.
     +   .NOT. HEADER_FLAG .AND. .NOT. NOHEADER_FLAG) THEN
	   NOHEADER_LFLAG = .TRUE.
	   HEADER_LFLAG = .FALSE.
	ELSE
	   HEADER_LFLAG = .FALSE.
	   HEADER_LFLAG = .FALSE.
	END IF
C
	IF (CLI$PRESENT('LETTER') .EQ. %LOC(CLI$_LOCPRES) .AND. .NOT.
     +   LETTER_FLAG .AND. .NOT. DRAFT_FLAG .AND. FORM_FLAG) THEN
	   IF (DRAFT_LFLAG .AND. LA100_FLAG) THEN
	      TYPE *,'%USERPRINT-E-CONFQUAL, Conflicting qualifiers,',
     +         ' /DRAFT/LETTER/'
	      SKIP_PARAM = .TRUE.
	      GO TO 500
	   ELSE IF (LA100_FLAG) THEN
              CALL BUILD_FORMAT (LPREFIX_STR,LETTER_MODE,LPREFIX_END_PT) 
	   ELSE
	      TYPE *, '%USERPRINT-I-NOLETSUP,',
     +         ' device does not support LETTER quality printing'
	   END IF
	END IF
C
	IF (CLI$PRESENT('PAGE_COUNT') .EQ. %LOC(CLI$_LOCPRES)) THEN
	   CALL CLI$GET_VALUE ('PAGE_COUNT',PAGE_COUNT_STR)
	   CALL CHAR2INT (PAGE_COUNT_STR,VALUE,ERROR_FLAG)
	   IF (ERROR_FLAG .OR. VALUE .LT. 1) THEN
	      TYPE *, '%USERPRINT-E-PAGCNTVAL,',
     +         ' Improper PAGE_COUNT syntax, or value out of range'
	      PAGE_COUNT_LFLAG = .FALSE.
	      SKIP_PARAM = .TRUE.
	      GO TO 500
	   END IF
	   PAGE_COUNT_LFLAG = .TRUE.
	ELSE
	   PAGE_COUNT_LFLAG = .FALSE.
	END IF
C
	IF (CLI$PRESENT('PITCH') .EQ. %LOC(CLI$_LOCPRES) .AND. .NOT.
     +   PITCH_FLAG .AND. FORM_FLAG) THEN
	   CALL CLI$GET_VALUE ('PITCH',PITCH_STR)
	   PITCH_LFLAG = .TRUE.
	   IF (PITCH_STR .EQ. '5') THEN
	      CALL BUILD_FORMAT (LPREFIX_STR,HP5,LPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 1
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '5'
	   ELSE IF (PITCH_STR .EQ. '6') THEN
	      CALL BUILD_FORMAT (LPREFIX_STR,HP6,LPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 1
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '6'
	   ELSE IF (PITCH_STR .EQ. '6.6') THEN
	      CALL BUILD_FORMAT (LPREFIX_STR,HP6_6,LPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 1
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '6'
	   ELSE IF (PITCH_STR .EQ. '8.25') THEN
	      CALL BUILD_FORMAT (LPREFIX_STR,HP8_25,LPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 1
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '8'
	   ELSE IF (PITCH_STR .EQ. '10') THEN
	      CALL BUILD_FORMAT (LPREFIX_STR,HP10,LPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 2
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '10'
	   ELSE IF (PITCH_STR .EQ. '12') THEN
	      CALL BUILD_FORMAT (LPREFIX_STR,HP12,LPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 2
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '12'
	   ELSE IF (PITCH_STR .EQ. '13.2') THEN
	      CALL BUILD_FORMAT (LPREFIX_STR,HP13_2,LPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 2
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '13'
	   ELSE IF (PITCH_STR .EQ. '16.5') THEN
	      CALL BUILD_FORMAT (LPREFIX_STR,HP16_5,LPREFIX_END_PT)
	      DEF_MARGIN_STR_LEN = 1
	      DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '8'
	   ELSE
	      TYPE *, '%USERPRINT-E-PITCHVAL,',
     +         ' Improper PITCH syntax, or value out of range'
	      PITCH_LFLAG = .FALSE.
	      SKIP_PARAM = .TRUE.
	      GO TO 500
	   END IF
	ELSE IF (LA100_FLAG .AND..NOT. PITCH_FLAG .AND. FORM_FLAG) THEN
	   PITCH_LFLAG = .FALSE.
	   DEF_MARGIN_STR_LEN = 2
	   DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '10'
	ELSE IF (LA120_FLAG .AND..NOT. PITCH_FLAG .AND. FORM_FLAG) THEN
	   PITCH_LFLAG = .FALSE.
	   DEF_MARGIN_STR_LEN = 1
	   DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN) = '1'
	END IF
C
	IF (CLI$PRESENT('MARGIN') .EQ. %LOC(CLI$_LOCPRES) .AND. .NOT.
     +   MARGIN_FLAG .AND. FORM_FLAG) THEN
	   CALL CLI$GET_VALUE ('MARGIN',MARGIN_STR)
	   CALL CHAR2INT (MARGIN_STR,VALUE,ERROR_FLAG)
	   IF (VALUE .GT. 80 .OR. ERROR_FLAG) THEN
	      TYPE *, '%USERPRINT-E-MARGINVAL,',
     +         ' Improper MARGIN syntax, or value out of range'
	      SKIP_PARAM = .TRUE.
	      MARGIN_LFLAG = .FALSE.
	      GO TO 500
	   ELSE IF (VALUE .LE. 0) THEN
	      MARGIN_STR(1:2) = '1 '
	   END IF
	   MARGIN_STR_LEN = INDEX(MARGIN_STR,' ') - 1
	   MARGIN_PREFIX(3:3+MARGIN_STR_LEN) =
     +      MARGIN_STR(1:MARGIN_STR_LEN)//'s'
	   CALL BUILD_FORMAT (LPREFIX_STR,
     +      MARGIN_PREFIX(1:3+MARGIN_STR_LEN),LPREFIX_END_PT)
	   MARGIN_LFLAG = .TRUE.
	ELSE
	   MARGIN_LFLAG = .FALSE.
	END IF
C
	IF (CLI$PRESENT('SPACE') .EQ. %LOC(CLI$_LOCPRES) .AND. .NOT.
     +   SPACE_FLAG) THEN
     	   SPACE_LFLAG = .TRUE.
	ELSE
	   SPACE_LFLAG = .FALSE.
	END IF
C
C---------Unwind the parameter list (in the case that it contains wild-
C         card entries), to determine specific file specifications.
C	  Keep track of default devices and directories.
C
	   UNWINDING = .TRUE.
	   GOOD_PARAMS = .FALSE.
C
	   DO WHILE (UNWINDING)
C
	      STATUS = LIB$FIND_FILE
     +         (PARAM_ITEM,RESULT_FS,CONTEXT,DEFAULT_FS,RELATED_FS)
C
	      IF (CONTEXT .EQ. 0) THEN
	         UNWINDING = .FALSE.
	         GO TO 500
	      END IF
C
	      RELATED_FS = RESULT_FS
	      FULL_FS_LEN = MAX( (INDEX(RESULT_FS,' ')-1), 1)
	      FULL_FILESPEC = RESULT_FS
	      I = MAX( (INDEX(RESULT_FS,';')-1), 1)
	      DEFAULT_FS = RESULT_FS(1:I)
	      I = INDEX(RESULT_FS,']') + 1
	      SHORT_FILESPEC = RESULT_FS(I:)
	      SHORT_FS_LEN = MAX( (INDEX(SHORT_FILESPEC,' ')-1), 1)
	      I = INDEX(SHORT_FILESPEC,'.') + 1
	      FILE_TYPE = SHORT_FILESPEC(I:I+2)
C
C---------See if the input file is there, and valid to print out. If not,
C         unwind the next file from current parameter item. Serious
C         errors cause aborting of this parameter item, and dequeueing 
C         of the next. (Note: This test is rather simple-minded... as
C	  long as the file isn't of an illegal type, it will go through.
C	  No better technique has been found. Clever solutions tend to
C	  back-fire when new or seldom used utilities create odd file
C	  attributes which aren't anticipated (and so get erroneously
C	  ruled as illegal) but must be accepted as valid.
C
	   INQUIRE (FILE=FULL_FILESPEC(1:FULL_FS_LEN),RECL=REC_LEN,
     +      EXIST=FILE_EXISTS,IOSTAT=IOSTAT)
C
	   IF (.NOT. FILE_EXISTS) THEN
	      TYPE *, '%USERPRINT-W-FILNOTFOU, Cannot access ',
     +         FULL_FILESPEC(1:FULL_FS_LEN),' as input'
	      GO TO 500
	   ELSE IF (FILE_TYPE .EQ. 'EXE' .OR. FILE_TYPE .EQ. 'TSK' .OR. 
     +       FILE_TYPE .EQ. 'OBJ' .OR. FILE_TYPE .EQ. 'STB' .OR.
     +       FILE_TYPE .EQ. 'SYS' .OR. FILE_TYPE .EQ. 'DIR' .OR.
     +       FILE_TYPE .EQ. 'PAR' .OR. FILE_TYPE .EQ. 'DIC' .OR.
     +       FILE_TYPE .EQ. 'MAI' .OR. FILE_TYPE .EQ. 'HLB' .OR.
     +       FILE_TYPE .EQ. 'OLB' .OR. FILE_TYPE .EQ. 'TLB' .OR.
     +       FILE_TYPE .EQ. 'MLB' .OR. FILE_TYPE .EQ. 'SML' .OR.
     +       FILE_TYPE .EQ. 'PIC' .OR. FILE_TYPE .EQ. 'PLV') THEN
	      TYPE *, '%USERPRINT-W-NOTTXTFIL, ',
     +         FULL_FILESPEC(1:FULL_FS_LEN), ' is not a text file'
	      IF (.NOT. FORM_FLAG) THEN
	         UNWINDING = .FALSE.
	         SKIP_PARAM = .TRUE.
	      END IF
	      GO TO 500
	   END IF
C
C---------If a Control-Y Exit-Flag, quit dequeueing files, and print
C	what we have.
C
	   IF (CTRLY_EXIT_FLAG) THEN
	      MORE_FILES = .FALSE.
	      UNWINDING = .FALSE.
	      GO TO 500
	   END IF
C
C---------If formatted, then copy the files over to the system print
C         directory with the selected options embedded in the code.
C         If unformatted, then leave files alone, and don't copy them
C         anywhere.
C
	   IF (.NOT. FORM_FLAG) THEN
	      GOOD_PARAMS = .TRUE.
	      GO TO 500
C
	   ELSE
	      OPEN (UNIT=1,FILE=FULL_FILESPEC(1:FULL_FS_LEN),
     +         STATUS='OLD',SHARED,READONLY,IOSTAT=IOSTAT)
C
	      IF (IOSTAT .EQ. FOR$IOS_FILNOTFOU) THEN
	         TYPE *, '%USERPRINT-W-FILNOTFOU, Cannot access ',
     +            FULL_FILESPEC(1:FULL_FS_LEN),' as input'
	         GO TO 500
	      ELSE IF (IOSTAT .EQ. FOR$IOS_OPEFAI) THEN
	         TYPE *, '%USERPRINT-W-FILOPENFAIL, Cannot open file ',
     +            FULL_FILESPEC(1:FULL_FS_LEN),' as input'
	         GO TO 500
	      ELSE IF (IOSTAT .GT. 0) THEN
	         TYPE *, '%USERPRINT-W-UNKERR, Unknown error opening ',
     +            FULL_FILESPEC(1:FULL_FS_LEN), ' as input'
	         UNWINDING = .FALSE.
	         GO TO 500
	      END IF
C
C---------Attempt to open the output file with the same version number
C         as the input file. If an output file already there, then let
C         the user know of the duplicate filespec, and skip over it.
C         Unknown errors cause unwinding of the next file from current
C         parameter item.
C
	      OPEN (UNIT=2,FILE=SHORT_FILESPEC(1:SHORT_FS_LEN),
     +         DEFAULTFILE='SYS$SYSPRINT:',STATUS='NEW',IOSTAT=IOSTAT,
     +         CARRIAGECONTROL='LIST',RECL=400)
C
	      IF (IOSTAT .EQ. FOR$IOS_OPEFAI) THEN
	         TYPE *, '%USERPRINT-W-DUPLFILE, duplicate version of ',
     +            FULL_FILESPEC(1:FULL_FS_LEN),
     +            ' already queued; unpredictable results'
	          GOOD_PARAMS = .TRUE.
	          CLOSE (1)
	          GO TO 500
	      END IF
C
	      IF (IOSTAT .GT. 0) THEN
	         TYPE *, '%USERPRINT-W-UNKERR, Unknown error opening ',
     +            FULL_FILESPEC(1:FULL_FS_LEN),' as output'
	         CLOSE (1)
	         UNWINDING = .FALSE.
	         GOTO 500
	      END IF
C
	      TYPE *, '  Formatting file ',
     +         FULL_FILESPEC(1:FULL_FS_LEN)
C
C---------Set up current margin value.
C
	      IF (MARGIN_FLAG .OR. MARGIN_LFLAG) THEN
	         ACT_MARGIN_STR_LEN = INDEX(MARGIN_STR,' ') - 1
	         ACTIVE_MARGIN_STR(1:ACT_MARGIN_STR_LEN) =
     +            MARGIN_STR(1:ACT_MARGIN_STR_LEN)
	      ELSE
	         ACT_MARGIN_STR_LEN = DEF_MARGIN_STR_LEN
	         ACTIVE_MARGIN_STR(1:ACT_MARGIN_STR_LEN) =
     +            DEFAULT_MARGIN_STR(1:ACT_MARGIN_STR_LEN)
	      END IF
C
	      CALL CHAR2INT (ACTIVE_MARGIN_STR(1:ACT_MARGIN_STR_LEN),
     +                       ACT_MARGIN,ERROR_FLAG)
	      CALL CHAR2INT (DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN),
     +                       DEF_MARGIN,ERROR_FLAG)
C
C---------If pitch is not explicitly specified, optimize horizontal
C         pitch so that maximum record lengths match page size. (This
C         may require shifting from LETTER mode to DRAFT mode if pitch
C         needed isn't supported by the generic font-pitch.) In all
C         cases, append on the appropriate active margin. (NOTE: This
C         algorithm is based on the maximum record length, rather than
C	  on actual printed length, and so is fooled by special
C         characters which cause the printed length to be different than
C         record length, such as tab, backspace, control characters,
C         etc.) The draft parameter must be specified before pitch and
C	  margins are declared. A default font-pitch of 10 is assumed
C	  if none is specified (as in the case of LA120's). We
C	  must keep track of current active margin settings for this
C	  file during any necessary auto-sizing that takes place, so
C 	  that tab settings can be calculated properly. 
C
	      CALL CLEAR_FORMAT (FPREFIX_STR,FPREFIX_END_PT,20)
C
	      FONT_10 = FONT_10_FLAG
	      FONT_12 = FONT_12_FLAG
	      IF (.NOT. FONT_10 .AND. .NOT. FONT_12) FONT_10 = .TRUE.

C	Try to fit the text onto the page with the specified margin
C	for 10-pitch fonts:
C
	      IF (.NOT. PITCH_FLAG .AND. .NOT. PITCH_LFLAG
     +         .AND. FONT_10) THEN
	         IF (FLOAT(REC_LEN+ACT_MARGIN)/10. .LE. 8.0) THEN
	            CALL BUILD_FORMAT
     +                (FPREFIX_STR,HP10,FPREFIX_END_PT)
	            CALL BUILD_FORMAT (FPREFIX_STR,ESCAPE//'['//
     +               ACTIVE_MARGIN_STR(1:ACT_MARGIN_STR_LEN)//'s',
     +               FPREFIX_END_PT)
C
C	Next try to fit the text onto the page with the default margin
C	for 10-pitch fonts:
C
	         ELSE IF (FLOAT(REC_LEN+DEF_MARGIN)/10. .LE. 8.0) THEN
	            CALL BUILD_FORMAT
     +                (FPREFIX_STR,HP10,FPREFIX_END_PT)
	            CALL BUILD_FORMAT (FPREFIX_STR,ESCAPE//'['//
     +               DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN)//'s',
     +               FPREFIX_END_PT)
	            ACT_MARGIN = DEF_MARGIN
	            ACT_MARGIN_STR_LEN = DEF_MARGIN_STR_LEN
	            ACTIVE_MARGIN_STR(1:ACT_MARGIN_STR_LEN) =
     +               DEFAULT_MARGIN_STR(1:ACT_MARGIN_STR_LEN)
		 ELSE
		    FONT_10 = .FALSE.
		 END IF
	      END IF
C
C	Try to fit the text onto the page with the specified margin
C	for 12-pitch fonts:
C
	      IF (LA100_FLAG) DEF_MARGIN = 12
C
	      IF (.NOT. PITCH_FLAG .AND. .NOT. PITCH_LFLAG
     +         .AND. FONT_12) THEN
	         IF (FLOAT(REC_LEN+ACT_MARGIN)/12. .LE. 8.0) THEN
	            CALL BUILD_FORMAT
     +                (FPREFIX_STR,HP12,FPREFIX_END_PT)
	            CALL BUILD_FORMAT (FPREFIX_STR,ESCAPE//'['//
     +               ACTIVE_MARGIN_STR(1:ACT_MARGIN_STR_LEN)//'s',
     +               FPREFIX_END_PT)
C
C	Next try to fit the text onto the page with the default margin
C	for 12-pitch fonts:
C
	         ELSE IF (FLOAT(REC_LEN+DEF_MARGIN)/12. .LE. 8.0) THEN
	            CALL BUILD_FORMAT
     +                (FPREFIX_STR,HP12,FPREFIX_END_PT)
		    IF (LA100_FLAG) THEN
			DEFAULT_MARGIN_STR(1:2) = '12'
			DEF_MARGIN_STR_LEN = 2
		    END IF
	            CALL BUILD_FORMAT (FPREFIX_STR,ESCAPE//'['//
     +               DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN)//'s',
     +               FPREFIX_END_PT)
	            ACT_MARGIN = DEF_MARGIN
	            ACT_MARGIN_STR_LEN = DEF_MARGIN_STR_LEN
	            ACTIVE_MARGIN_STR(1:ACT_MARGIN_STR_LEN) =
     +               DEFAULT_MARGIN_STR(1:ACT_MARGIN_STR_LEN)
		 ELSE
		    FONT_12 = .FALSE.
		 END IF
	      END IF
C
C	The lines are too long to fit the page for the generic font
C	pitch chosen, so we will need to pick a more condensed pitch,
C	and go to draft-mode if letter-mode has been set.
C
	      IF (.NOT. PITCH_FLAG .AND. .NOT. PITCH_LFLAG
     +         .AND. .NOT. FONT_10 .AND. .NOT. FONT_12) THEN
C
	         IF (FLOAT(REC_LEN+DEF_MARGIN)/12. .LE. 8.0) THEN
	            IF (LETTER_FLAG) THEN
		       CALL BUILD_FORMAT 
     +                  (FPREFIX_STR,DRAFT_MODE,FPREFIX_END_PT)
                       TYPE *, '%USERPRINT-I-MODPITCH, ',
     +                  'setting draft mode and reducing pitch on ',
     +                  'above document' 
		    END IF
	            CALL BUILD_FORMAT
     +                (FPREFIX_STR,HP12,FPREFIX_END_PT)
		    IF (LA100_FLAG) THEN
			DEFAULT_MARGIN_STR(1:2) = '12'
			DEF_MARGIN_STR_LEN = 2
		    END IF
	            CALL BUILD_FORMAT (FPREFIX_STR,ESCAPE//'['//
     +               DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN)//'s',
     +               FPREFIX_END_PT)
	            ACT_MARGIN = DEF_MARGIN
	            ACT_MARGIN_STR_LEN = DEF_MARGIN_STR_LEN
	            ACTIVE_MARGIN_STR(1:ACT_MARGIN_STR_LEN) =
     +               DEFAULT_MARGIN_STR(1:ACT_MARGIN_STR_LEN)
C
	         ELSE IF (FLOAT(REC_LEN+DEF_MARGIN)/13.2 .LE. 8.0) THEN
	            IF (LETTER_FLAG) THEN
		       CALL BUILD_FORMAT 
     +                  (FPREFIX_STR,DRAFT_MODE,FPREFIX_END_PT)
                       TYPE *, '%USERPRINT-I-MODPITCH, ',
     +                  'setting draft mode and reducing pitch on ',
     +                  'above document' 
		    END IF
	            CALL BUILD_FORMAT
     +                (FPREFIX_STR,HP13_2,FPREFIX_END_PT)
		    IF (LA100_FLAG) THEN
			DEFAULT_MARGIN_STR(1:2) = '13'
			DEF_MARGIN_STR_LEN = 2
			DEF_MARGIN = 13
		    END IF
	            CALL BUILD_FORMAT (FPREFIX_STR,ESCAPE//'['//
     +               DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN)//'s',
     +               FPREFIX_END_PT)
	            ACT_MARGIN = DEF_MARGIN
	            ACT_MARGIN_STR_LEN = DEF_MARGIN_STR_LEN
	            ACTIVE_MARGIN_STR(1:ACT_MARGIN_STR_LEN) =
     +               DEFAULT_MARGIN_STR(1:ACT_MARGIN_STR_LEN)
C
	         ELSE
	            IF (LETTER_FLAG) THEN
		       CALL BUILD_FORMAT 
     +                  (FPREFIX_STR,DRAFT_MODE,FPREFIX_END_PT)
                       TYPE *, '%USERPRINT-I-MODPITCH, ',
     +                  'setting draft mode and reducing pitch on ',
     +                  'above document' 
		    END IF
	            CALL BUILD_FORMAT
     +                (FPREFIX_STR,HP16_5,FPREFIX_END_PT)
		    IF (LA100_FLAG) THEN
			DEFAULT_MARGIN_STR(1:2) = '8'
			DEF_MARGIN_STR_LEN = 1
			DEF_MARGIN = 8
		    END IF
	            CALL BUILD_FORMAT (FPREFIX_STR,ESCAPE//'['//
     +               DEFAULT_MARGIN_STR(1:DEF_MARGIN_STR_LEN)//'s',
     +               FPREFIX_END_PT)
	            ACT_MARGIN = DEF_MARGIN
	            ACT_MARGIN_STR_LEN = DEF_MARGIN_STR_LEN
	            ACTIVE_MARGIN_STR(1:ACT_MARGIN_STR_LEN) =
     +               DEFAULT_MARGIN_STR(1:ACT_MARGIN_STR_LEN)
	         END IF
	      END IF
C
C	If a pitch was specified at either global or local level,
C	then add the sequence onto the file-prefix string.
C
	      IF (PITCH_FLAG .OR. PITCH_LFLAG)
     +	          CALL BUILD_FORMAT (FPREFIX_STR,ESCAPE//'['//
     +            ACTIVE_MARGIN_STR(1:ACT_MARGIN_STR_LEN)//'s',
     +            FPREFIX_END_PT)
C
C---------If a Control-Y Exit-Flag, close file opened, and queue what
C	there is (so that the files copied get deleted from
C	SYS$SYSPRINT: after being printed.) 
C
	IF (CTRLY_EXIT_FLAG) THEN
	   MORE_FILES = .FALSE.
	   UNWINDING = .FALSE.
	   GO TO 450
	END IF
C
C---------Build correct tab settings from active margin value.
C
	CALL BUILD_TAB (TPREFIX_STR,TPREFIX_END_PT,ACT_MARGIN)
C
C---------Copy the file over, embedding the prefix and suffix control
C         strings into the text file.
C	      
C         The first record has the default, global, and local prefix
C         strings inserted into the front end of the line.
C
	      READ (1,310,END=400) NUMBER,FILE_BUFFER
310	      FORMAT (Q,250A1)
C
	      WRITE (2,320)
     +         (SUFFIX_STR(I),  I=1,SUFFIX_END_PT),
     +         (GPREFIX_STR(I), I=1,GPREFIX_END_PT),
     +         (LPREFIX_STR(I), I=1,LPREFIX_END_PT),
     +         (FPREFIX_STR(I), I=1,FPREFIX_END_PT),
     +         (TPREFIX_STR(I), I=1,TPREFIX_END_PT), C_R,
     +         (FILE_BUFFER(I), I=1,NUMBER)
320	      FORMAT (50A1,50A1,50A1,20A1,55A1,1A1,250A1)
C
C         Now copy over the bulk of the file, until either end-of-file,
C	  or until someone decides to abort the job with a CTRL-Y.
C
	   DO WHILE (.NOT. CTRLY_EXIT_FLAG)
	      READ  (1,310,END=400) NUMBER,FILE_BUFFER
	      WRITE (2,360) (FILE_BUFFER(I), I=1,NUMBER)
360	      FORMAT (250A1)
	   END DO
C
C         Lastly, append the suffix strings to the end of the file.
C
400	      WRITE (2,410)
     +         (SUFFIX_STR(I), I=1,SUFFIX_END_PT),
     +         (TSUFFIX_STR(I), I=1,TSUFFIX_END_PT)
410	      FORMAT(50A1,55A1)
C
C         Close everything back up.
C
450	      CLOSE (UNIT=1)
	      CLOSE (UNIT=2)
	      GOOD_PARAMS = .TRUE.
	   END IF
C
500	   END DO
C
C---------If no valid files found during this parameter unwinding
C         session, then skip the print-command assembly procedure below,
C         and go directly back to the top of the loop to get the next
C         parameter. Else, set a flag denoting that at least one good 
C         entry has been made into the print command line.
C
	   IF (.NOT. GOOD_PARAMS .OR. SKIP_PARAM) THEN
	      TYPE *, '%USERPRINT-W-SKIPPARAM, ignoring entry item ',
     +        PARAM_ITEM(1:PARAM_ITEM_LEN),' as input'
	      GO TO 600
	   ELSE
	      COM_LINE_ENTRIES = .TRUE.
	   END IF
C
C---------If unformatted, append on the complete form of the parameter
C         entry. If formatted, append on the short form of the parameter
C         entry if any files were found and copied over to SYS$SYSPRINT:
C
	   IF (FORM_FLAG) THEN
              CALL BUILD_COM (COM_LINE,START_PT,END_PT,
     +         SHORT_PARAM(1:SHORT_PARAM_LEN),'* ')
	   ELSE
              CALL BUILD_COM (COM_LINE,START_PT,END_PT,
     +         PARAM_ITEM(1:PARAM_ITEM_LEN),'* ')
	   END IF
	   START_PT = START_PT - 1
	   END_PT = END_PT - 1
C
C---------Append on the local qualifiers for this parameter.
C
	   IF (COPIES_LFLAG)
     +      CALL BUILD_COM
     +       (COM_LINE,START_PT,END_PT,'/COP=',COPIES_STR)
C
	   IF (DELETE_LFLAG)
     +      CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','DEL ')
C
	   IF (FLAG_PAGE_LFLAG) THEN
              CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','FLA ')
	   ELSE IF (NOFLAG_PAGE_LFLAG) THEN
              CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','NOFLA ')
	   END IF
C
	   IF (HEADER_LFLAG) THEN
              CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','HEA ')
	   ELSE IF (NOHEADER_LFLAG) THEN
              CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','NOHEA ')
	   END IF
C
	   IF (PAGE_COUNT_LFLAG)
     +      CALL BUILD_COM
     +       (COM_LINE,START_PT,END_PT,'/PAGE=',PAGE_COUNT_STR)
C
	   IF (SPACE_LFLAG)
     +      CALL BUILD_COM (COM_LINE,START_PT,END_PT,'/','SP ')
C
C---------Append on a comma to the end of the last parameter item.
C
	   CALL BUILD_COM (COM_LINE,START_PT,END_PT,',',', ')
	   START_PT = START_PT - 1
	   END_PT = END_PT - 1
C
C---------Quit dequeueing files if exit-flag set by shutting off
C         MORE_FILES flag.
C
	IF (CTRLY_EXIT_FLAG) MORE_FILES = .FALSE.
C
600	END DO
C
C---------If no valid parameters were entered, then exit.
C
	IF (.NOT. COM_LINE_ENTRIES) THEN
	   TYPE *, '%USERPRINT-I-NOJOBSQUED,',
     +      ' No jobs were queued for printing'
	   CALL EXIT
	END IF
C
C---------Adjust END_PT to actual end.
C
	END_PT = END_PT - 1
C
C---------Create a file containing the command line, noting the full
C         filespec for it.
C
	OPEN (UNIT=2,FILE='SYS$SYSPRINT:USERPRINT.PRT',
     +   STATUS='NEW',IOSTAT=IOSTAT,CARRIAGECONTROL='LIST')
C
	INQUIRE (UNIT=2,NAME=FULL_FILESPEC,IOSTAT=IOSTAT)
	FULL_FS_LEN = MAX( (INDEX(FULL_FILESPEC,' ') - 1), 1)
C
	IF (END_PT .LE.  70) THEN
	   WRITE (2,810) COM_LINE(1:END_PT)
C
	ELSE IF (END_PT .LE. 140) THEN
	   WRITE (2,800) COM_LINE(1:70)
	   WRITE (2,810) COM_LINE(71:END_PT)
C
	ELSE IF (END_PT .LE. 210) THEN
	   WRITE (2,800) COM_LINE(1:70)
	   WRITE (2,800) COM_LINE(71:140)
	   WRITE (2,810) COM_LINE(141:END_PT)
C
	ELSE
	   WRITE (2,800) COM_LINE(1:70)
	   WRITE (2,800) COM_LINE(71:140)
	   WRITE (2,800) COM_LINE(141:210)
	   WRITE (2,810) COM_LINE(211:END_PT)
	END IF
800	FORMAT (A,'-')
810	FORMAT (A)
C
	CLOSE (2)
C
C---------Execute the command procedure to queue the jobs.
C
	CALL LIB$DO_COMMAND
     +   ('@SYS$SYSROOT:[SYSEXE.UTILITIES]USERPRINT '
     +    //FULL_FILESPEC(1:FULL_FS_LEN))
C
C---------Failure to execute the LIB$DO_COMMAND
C
	TYPE *, '%USERPRINT-F-LIBDOFAIL, command line not queued -',
     +   ' notify System Manager'
	CALL EXIT
	END
C
C
C=====================================================================
C
	SUBROUTINE BUILD_COM 
     +             (COM_LINE,START_PT,END_PT,TEXT_STR,VALUE_STR)
C
C=====================================================================
C
C
	CHARACTER  COM_LINE*(*),TEXT_STR*(*),VALUE_STR*(*)
	INTEGER*2  VALUE_LENGTH,START_PT,END_PT
C
C---------Append on the new text strings, and update the pointers.
C
	TEXT_LENGTH  = LEN(TEXT_STR)
	VALUE_LENGTH = INDEX(VALUE_STR,' ') - 1
	END_PT = END_PT + TEXT_LENGTH + VALUE_LENGTH
	COM_LINE(START_PT:END_PT) = 
     +     TEXT_STR(1:)//VALUE_STR(1:VALUE_LENGTH)
	START_PT = END_PT + 1
	RETURN
	END
C
C
C=====================================================================
C
	SUBROUTINE CHAR2INT (VALUE_STR,VALUE,ERROR_FLAG)
C
C=====================================================================
C
C
	CHARACTER  VALUE_STR*(*)
	INTEGER*2  VALUE
	LOGICAL*4  ERROR_FLAG
C
C---------Convert the input character string into an integer number.
C         Set the ERROR_FLAG if a conversion error occurs.
C
	ERROR_FLAG = .FALSE.
C
	READ (VALUE_STR,100,ERR=200,END=200) VALUE
100	FORMAT (BNI15)
	RETURN
C
200	ERROR_FLAG = .TRUE.
	VALUE = 0
	RETURN
	END
C
C
C=====================================================================
C
	SUBROUTINE BUILD_FORMAT (BYTE_STRING,TEXT_STR,END_PT)
C
C=====================================================================
C
C
	CHARACTER  TEXT_STR*(*)
	INTEGER*2  END_PT,I,LENGTH
	BYTE       BYTE_STRING(1)
C
	DO I=1,LEN(TEXT_STR)
	   END_PT = END_PT + 1
	   BYTE_STRING(END_PT) = ICHAR(TEXT_STR(I:I))
	END DO
	RETURN
C
C
	ENTRY CLEAR_FORMAT (BYTE_STRING,END_PT,LENGTH)
C
	DO I=1,LENGTH
	   BYTE_STRING(I) = 0
	END DO
	END_PT = 0
	RETURN
	END
C
C
C=====================================================================
C
	SUBROUTINE BUILD_TAB (BYTE_STR,END_PT,MARGIN)
C
C=====================================================================
C
C
	INTEGER*2  END_PT,MARGIN,TAB_COUNT,TAB_VALUE
	CHARACTER  CHAR_STR*3
	BYTE       BYTE_STR(1)
	PARAMETER  ESCAPE  = 27
	PARAMETER  BRACKET = ICHAR('[')
	PARAMETER  SEMI_COL= ICHAR(';')
	PARAMETER  SMALL_U = ICHAR('u')
C
	BYTE_STR(1) = ESCAPE
	BYTE_STR(2) = BRACKET
	END_PT      = 2
	TAB_COUNT   = 0
	TAB_VALUE   = MARGIN + 8
C
	DO WHILE (TAB_COUNT .LE. 16 .AND. TAB_VALUE .LE. 132)
	   TAB_COUNT = TAB_COUNT + 1
	   WRITE (CHAR_STR,100) TAB_VALUE
100	   FORMAT(BNI3)
C
	   IF (TAB_VALUE .LT. 10) THEN
	      BYTE_STR(END_PT+1) = ICHAR(CHAR_STR(3:3))
	      END_PT = END_PT + 2
C
	   ELSE IF (TAB_VALUE .LT. 100) THEN
	      BYTE_STR(END_PT+1) = ICHAR(CHAR_STR(2:2))
	      BYTE_STR(END_PT+2) = ICHAR(CHAR_STR(3:3))
	      END_PT = END_PT + 3
C
	   ELSE
	      BYTE_STR(END_PT+1) = ICHAR(CHAR_STR(1:1))
	      BYTE_STR(END_PT+2) = ICHAR(CHAR_STR(2:2))
	      BYTE_STR(END_PT+3) = ICHAR(CHAR_STR(3:3))
	      END_PT = END_PT + 4
	   END IF
C
	   BYTE_STR(END_PT) = SEMI_COL
	   TAB_VALUE = TAB_VALUE + 8
	END DO
	BYTE_STR(END_PT) = SMALL_U
	RETURN
	END
C
C
C=====================================================================
C
	SUBROUTINE ENABLE_CTRLY
C
C=====================================================================
C
C
	EXTERNAL   CTRLY_HANDLER

	INTEGER*4  TERM_CHAN,STATUS,SYS$QIOW
	LOGICAL*1  FORM_FLAG,CTRLY_EXIT_FLAG
C
	INCLUDE    '($IODEF)'
C
	COMMON     /HANDLER/TERM_CHAN,FORM_FLAG,CTRLY_EXIT_FLAG
C
	STATUS = SYS$QIOW (,%VAL(TERM_CHAN),%VAL(IO$_SETMODE .OR. 
     +   IO$M_CTRLYAST),,,,CTRLY_HANDLER,,,,,)
	IF (.NOT. STATUS) THEN
	   TYPE *, '%USERPRINT-F-CTRLENAB, Cannot enable <CTRL-Y>'
	   CALL LIB$STOP(%VAL(STATUS))
	END IF
	RETURN
	END
C
C
C=====================================================================
C
	SUBROUTINE CTRLY_HANDLER
C
C=====================================================================
C
C
	INTEGER*4  TERM_CHAN,STATUS
	LOGICAL*1  FORM_FLAG,CTRLY_EXIT_FLAG
C
	INCLUDE    '($LIBCLIDEF)'
C
	COMMON     /HANDLER/TERM_CHAN,FORM_FLAG,CTRLY_EXIT_FLAG
C
C---------Re-enable the interupt handler.
C
	CALL ENABLE_CTRLY
C
C---------If unformatted, exit now; else, set the Control-Y Exit-Flag,
C	and let the program exit when it can.
C
	IF (FORM_FLAG) THEN
	   CTRLY_EXIT_FLAG = .TRUE.
	   RETURN
	ELSE
	   STATUS = LIB$ENABLE_CTRL (LIB$M_CLI_CTRLY,)
	   IF (STATUS .NE. 1) TYPE *,
     +      '%USERPRINT-W-NOCTRLYENAB, Failure to enable <CTRL-Y>'
	   CALL EXIT
	END IF
	END
