      Program SyncSecM
!
!     Synchronize system times between systems connected by shared memory.
!     Also sets Next Interval Count Register to a user-defined value.
!     MASTER process.
!
!     Written by:  Mark Paulk
!     System Development Corporation
!     21 September 1983
!

      include 'syncsec.cmn'

      character*24 system_time_string
      character*4 node_name(max_nodes)
     &/'VX01', 'VX02', 'VX03', 'VX04', 'VX05', 'VX06'/
      character*4 this_node
      logical node_list(max_nodes) /max_nodes*.false./
      logical sma_on /.false./, smb_on /.false./
      character*1 answer
      integer input_addr(2), output_addr(2)


!     Log onto the network to submit the other jobs to synch.

      status = d$signon ('$SYS')
      if (.not. status) call lib$signal(%val(status))

!     Find out where the program is running.

      status = sys$trnlog ('$PROCESSOR', , this_node, , , %val(6))
      if (.not. status) call lib$signal(%val(status))
      if (this_node .ne. 'VX03' .and. this_node .ne. 'VX04') then
         type *, this_node, ' is not connected to all systems.'
         write (6,60)
         read (5,20) j, answer
         call str$upcase (answer, answer)
         if (answer .eq. 'N') stop ' '
      end if

!     Input system time that systems are to be synchronized at and Next
!     Interval Count Register value (when 10 mil clock ticks in 100 ns).

      write (6,10)
 10   format ('$System time at synch: ')
      read (5,20) sts_len, system_time_string
 20   format (q,a)
      if (sts_len .eq. 0) then
         call sys$gettim (system_time)
      else
         call str$upcase (system_time_string, system_time_string)
         call sys$bintim (system_time_string(1:sts_len), system_time)
      end if
      write (6,30)
 30   format ('$NICR value (10,000 default)? ')
      read (5,40) nicr
 40   format (i)
      if (nicr .eq. 0) nicr = 10 000
      nicr = -nicr
      call sys$asctim (sts_len, system_time_string, system_time, )
      write (6,45) system_time_string(1:sts_len), nicr/-10000
 45   format (' Systems will synch at ', a/
     &' System time is running at ', i, ':1')

!     Find out which nodes are to be synched.

      do i=1,max_nodes
         if (this_node .eq. node_name(i)) then
            this_node_ptr = i
         else
            write (6,50) node_name(i)
 50         format ('$Synch ',a,'? ')
            read (5,20) j, answer
            call str$upcase (answer, answer)
            if (answer .eq. 'Y') then
               status = d$nodestatus (node_name(i))
               if (.not. status) then
                  type *, node_name(i),  ' is not up.'
                  write (6,60)
 60               format ('$Do you wish to continue? ')
                  read (5,20) j, answer
                  call str$upcase (answer, answer)
                  if (answer .eq. 'N') call lib$stop(%val(status))
               else
                  node_list(i) = .true.
                  status = d$submit (node_name(i), 'SYSTEM',
     &            'ARC$SYSTEM:SYNCSECS')
                  if (.not. status) call lib$signal(%val(status))
                  if (i .le. 3) then
                     sma_on = .true.
                  else
                     smb_on = .true.
                  end if
               end if
            end if
         end if
      end do

!     Define flag characteristics for the global section as Global,
!     Permanent, and Writeable.

      flags = SEC$M_GBL .or. SEC$M_PERM .or. SEC$M_WRT

!     Open the disk file for global section creation and mapping by calling
!     the bliss routine OPENSEC.
!     Define the input addresses.
!     Create and map a global section in shared memory.
!     Initialize the data in the shared memory - system time and NICR.

      if (sma_on) then
         call opensec ('SYNC.A', sec_size, channel_a)
         input_addr(1) = %loc(buffer_a(1))
         input_addr(2) = %loc(buffer_a(512))
         status = sys$crmpsc (input_addr, output_addr, , %val(flags),
     &   '$SMA:SYNC_A', , , %val(channel_a), %val(sec_size), , , )
         write (6,70) input_addr, output_addr
 70      format (' Shared memory A - Specified:', 2z8,
     &   ' Obtained:', 2z8)
         if(.not. status) call lib$stop(%val(status))
         system_time_a(1) = system_time(1)
         system_time_a(2) = system_time(2)
         nicr_a = nicr
      end if

      if (smb_on) then
         call opensec ('SYNC.B', sec_size, channel_b)
         input_addr(1) = %loc(buffer_b(1))
         input_addr(2) = %loc(buffer_b(512))
         status = sys$crmpsc (input_addr, output_addr, , %val(flags),
     &   '$SMB:SYNC_B', , , %val(channel_b), %val(sec_size), , , )
         write (6,80) input_addr, output_addr
 80      format (' Shared memory B - Specified:', 2z8,
     &   ' Obtained:', 2z8)
         if(.not. status) call lib$stop(%val(status))
         system_time_b(1) = system_time(1)
         system_time_b(2) = system_time(2)
         nicr_b = nicr
      end if

!     Associate with common event flag clusters.

      if (sma_on) then
         status = sys$ascefc (%val(event_flag_a), '$SMA:SYNCEFNA', , )
         if (.not. status) call lib$stop(%val(status))
      end if

      if (smb_on) then
         status = sys$ascefc (%val(event_flag_b), '$SMB:SYNCEFNB', , )
         if (.not. status) call lib$stop(%val(status))
      end if

!     Wait for all partners to begin.

      do i=1,max_nodes
         if (node_list(i)) then
            if (i .le. 3) then
               efn = event_flag_a + i
            else
               efn = event_flag_b + i
            end if
            status = sys$waitfr (%val(efn))
            if (.not. status) call lib$signal(%val(status))
            type *, node_name(i), ' checked in.'
         end if
      end do

!     Notify partners to stop their clocks and synch their times.

      if (sma_on) then
         status = sys$setef (%val(event_flag_a))
         if (.not. status) call lib$signal(%val(status))
      end if

      if (smb_on) then
         status = sys$setef (%val(event_flag_b))
         if (.not. status) call lib$signal(%val(status))
      end if

!     The following write should induce a small delay which
!     will insure that the slaves have synched by the time
!     of the go ahead.

      type *, 'Ready to synchronize clocks.'

!     Call kernel mode routines to stop the clock, synch our
!     time, and give the go ahead to the other routines to restart.

      call stop_icr
      status = set_nicr (nicr)
      if (.not. status) call lib$signal(%val(status))
      status = set_system_time (system_time)
      if (.not. status) call lib$signal(%val(status))
      if (sma_on) call sys$setef (%val(go_ahead_efn_a))
      if (smb_on) call sys$setef (%val(go_ahead_efn_b))
      call start_icr

!     Begin cleanup by deassociating from the event flag cluster, deassigning
!     the I/O channel to the file, and finally marking the global section for
!     deletion.

      if (sma_on) then
         status = sys$dacefc(%val(event_flag_a))
         status = sys$dassgn(%val(channel_a))
         status = sys$dgblsc(,'$SMA:SYNC_A',)
      end if

      if (smb_on) then
         status = sys$dacefc(%val(event_flag_b))
         status = sys$dassgn(%val(channel_b))
         status = sys$dgblsc(,'$SMB:SYNC_B',)
      end if

      stop
      end
